# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Provides `image_as_moving_sequence`."""

import collections

import tensorflow.compat.v2 as tf


def _create_moving_sequence(image, pad_lefts, total_padding):
  """Create a moving image sequence from the given image a left padding values.

  Args:
    image: [in_h, in_w, n_channels] uint8 array
    pad_lefts: [sequence_length, 2] int32 array of left padding values
    total_padding: tensor of padding values, (pad_h, pad_w)

  Returns:
    [sequence_length, out_h, out_w, n_channels] uint8 image sequence, where
      out_h = in_h + pad_h, out_w = in_w + out_w
  """

  with tf.name_scope("moving_sequence"):
    def get_padded_image(args):
      pad_left, = args
      pad_right = total_padding - pad_left
      padding = tf.stack([pad_left, pad_right], axis=-1)
      z = tf.zeros((1, 2), dtype=pad_left.dtype)
      padding = tf.concat([padding, z], axis=0)
      return tf.pad(image, padding)

    padded_images = tf.map_fn(
        get_padded_image, [pad_lefts], dtype=tf.uint8, infer_shape=False,
        back_prop=False)

  return padded_images


def _get_linear_trajectory(x0, velocity, t):
  """Construct a linear trajectory from x0.

  Args:
    x0: N-D float tensor.
    velocity: N-D float tensor
    t: [sequence_length]-length float tensor

  Returns:
    x: [sequence_length, ndims] float tensor.
  """
  x0 = tf.convert_to_tensor(x0)
  velocity = tf.convert_to_tensor(velocity)
  t = tf.convert_to_tensor(t)
  if x0.shape.ndims != 1:
    raise ValueError("x0 must be a rank 1 tensor")
  if velocity.shape.ndims != 1:
    raise ValueError("velocity must be a rank 1 tensor")
  if t.shape.ndims != 1:
    raise ValueError("t must be a rank 1 tensor")
  x0 = tf.expand_dims(x0, axis=0)
  velocity = tf.expand_dims(velocity, axis=0)
  dx = velocity * tf.expand_dims(t, axis=-1)
  linear_trajectories = x0 + dx
  assert linear_trajectories.shape.ndims == 2, \
    "linear_trajectories should be a rank 2 tensor"
  return linear_trajectories


def _bounce_to_bbox(points):
  """Bounce potentially unbounded points to [0, 1].

  Bouncing occurs by exact reflection, i.e. a pre-bound point at 1.1 is moved
  to 0.9, -0.2 -> 0.2. This theoretically can occur multiple times, e.g.
  2.3 -> -0.7 -> 0.3

  Implementation
  points <- points % 2
  return min(2 - points, points)

  Args:
    points: float array

  Returns:
    tensor with same shape/dtype but values in [0, 1].
  """
  points = points % 2
  return tf.math.minimum(2 - points, points)


def _get_random_unit_vector(ndims=2, dtype=tf.float32):
  x = tf.random.normal((ndims,), dtype=dtype)
  return x / tf.linalg.norm(x, axis=-1, keepdims=True)

MovingSequence = collections.namedtuple(
    "_MovingSequence",
    ["image_sequence", "trajectory", "start_position", "velocity"])


def image_as_moving_sequence(
    image, sequence_length=20, output_size=(64, 64), velocity=0.1,
    start_position=None):
  """Turn simple static images into sequences of the originals bouncing around.

  Adapted from Srivastava et al.
  http://www.cs.toronto.edu/~nitish/unsupervised_video/

  Example usage:
  ```python
  import tensorflow.compat.v2 as tf
  import tensorflow_datasets as tfds
  from tensorflow_datasets.video import moving_sequence
  tf.enable_v2_behavior()

  def animate(sequence):
    import numpy as np
    import matplotlib.pyplot as plt
    import matplotlib.animation as animation
    sequence = np.squeeze(sequence, axis=-1)

    fig = plt.figure()
    plt.axis("off")
    ims = [[plt.imshow(im, cmap="gray", animated=True)] for im in sequence]
    # don't remove `anim =` as linter may suggets
    # weird behaviour, plot will freeze on last frame
    anim = animation.ArtistAnimation(
        fig, ims, interval=50, blit=True, repeat_delay=100)

    plt.show()
    plt.close()


  tf.enable_v2_behavior()
  mnist_ds = tfds.load("mnist", split=tfds.Split.TRAIN, as_supervised=True,
                       shuffle_files=True)
  mnist_ds = mnist_ds.repeat().shuffle(1024)

  def map_fn(image, label):
    sequence = moving_sequence.image_as_moving_sequence(
        image, sequence_length=20)
    return sequence.image_sequence

  moving_mnist_ds = mnist_ds.map(map_fn).batch(2).map(
      lambda x: dict(image_sequence=tf.reduce_max(x, axis=0)))

  # # for comparison with test data provided by original authors
  # moving_mnist_ds = tfds.load("moving_mnist", split=tfds.Split.TEST)

  for seq in moving_mnist_ds:
    animate(seq["image_sequence"].numpy())
  ```

  Args:
    image: [in_h, in_w, n_channels] tensor defining the sub-image to be bouncing
        around.
    sequence_length: int, length of sequence.
    output_size: (out_h, out_w) size returned images.
    velocity: scalar speed or 2D velocity of image. If scalar, the 2D
        velocity is randomly generated with this magnitude. This is the
        normalized distance moved each time step by the sub-image, where
        normalization occurs over the feasible distance the sub-image can move
        e.g if the input image is [10 x 10] and the output image is [60 x 60],
        a speed of 0.1 means the sub-image moves (60 - 10) * 0.1 = 5 pixels per
        time step.
    start_position: 2D float32 normalized initial position of each
        image in [0, 1]. Randomized uniformly if not given.

  Returns:
    `MovingSequence` namedtuple containing:
        `image_sequence`:
          [sequence_length, out_h, out_w, n_channels] image at each time step.
          padded values are all zero. Same dtype as input image.
        `trajectory`: [sequence_length, 2] float32 in [0, 1]
          2D normalized coordinates of the image at every time step.
        `start_position`: 2D float32 initial position in [0, 1].
          2D normalized initial position of image. Same as input if provided,
          otherwise the randomly value generated.
        `velocity`: 2D float32 normalized velocity. Same as input velocity
          if provided as a 2D tensor, otherwise the random velocity generated.
  """
  ndims = 2
  image = tf.convert_to_tensor(image)
  if image.shape.ndims != 3:
    raise ValueError("image must be rank 3, got %s" % str(image))
  output_size = tf.TensorShape(output_size)
  if len(output_size) != ndims:
    raise ValueError("output_size must have exactly %d elements, got %s"
                     % (ndims, output_size))
  image_shape = tf.shape(image)
  if start_position is None:
    start_position = tf.random.uniform((ndims,), dtype=tf.float32)
  elif start_position.shape != (ndims,):
    raise ValueError("start_positions must (%d,)" % ndims)
  velocity = tf.convert_to_tensor(velocity, dtype=tf.float32)
  if velocity.shape.ndims == 0:
    velocity = _get_random_unit_vector(ndims, tf.float32) * velocity
  elif velocity.shape.ndims != 1:
    raise ValueError("velocity must be rank 0 or rank 1, got %s" % velocity)
  t = tf.range(sequence_length, dtype=tf.float32)
  trajectory = _get_linear_trajectory(start_position, velocity, t)
  trajectory = _bounce_to_bbox(trajectory)

  total_padding = output_size - image_shape[:2]

  if not tf.executing_eagerly():
    cond = tf.compat.v1.assert_greater(total_padding, -1)
    with tf.control_dependencies([cond]):
      total_padding = tf.identity(total_padding)

  sequence_pad_lefts = tf.cast(
      tf.math.round(trajectory * tf.cast(total_padding, tf.float32)), tf.int32)

  sequence = _create_moving_sequence(image, sequence_pad_lefts, total_padding)
  sequence.set_shape(
      [sequence_length] + output_size.as_list() + [image.shape[-1]])
  return MovingSequence(
      image_sequence=sequence,
      trajectory=trajectory,
      start_position=start_position,
      velocity=velocity)



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""RoboNet dataset.

RoboNet: Large-Scale Multi-Robot Learning

Sudeep Dasari, Frederik Ebert, Stephen Tian, Suraj Nair, Bernadette Bucher,
Karl Schmeckpeper, Siddharth Singh, Sergey Levine, Chelsea Finn

https://www.robonet.wiki/
"""

import os
import textwrap

import numpy as np
import tensorflow.compat.v2 as tf

import tensorflow_datasets.public_api as tfds


DATA_URL_SAMPLE = ('https://drive.google.com/uc?export=download&'
                   'id=1YX2TgT8IKSn9V4wGCwdzbRnS53yicV2P')
DATA_URL = ('https://drive.google.com/uc?export=download&'
            'id=1BkqHzfRkfzgzCfc73NbNnPMK_rg3i1n9')

STATES_DIM = 5
ACTIONS_DIM = 5

_CITATION = """\
@article{dasari2019robonet,
  title={RoboNet: Large-Scale Multi-Robot Learning},
  author={Dasari, Sudeep and Ebert, Frederik and Tian, Stephen and
  Nair, Suraj and Bucher, Bernadette and Schmeckpeper, Karl
  and Singh, Siddharth and Levine, Sergey and Finn, Chelsea},
  journal={arXiv preprint arXiv:1910.11215},
  year={2019}
}
"""


class RobonetConfig(tfds.core.BuilderConfig):
  """"Configuration for RoboNet video rescaling."""

  def __init__(
      self, *, sample_dataset=False, width=None, height=None, **kwargs
  ):
    """The parameters specifying how the dataset will be processed.

    The dataset comes with three separate splits. You can specify which split
    you want in `split_number`. If `width` and `height` are set, the videos
    will be rescaled to have those heights and widths (using ffmpeg).

    Args:
      sample_dataset: Whether or not to use the sample dataset.
      width: An integer with the width or None.
      height: An integer with the height or None.
      **kwargs: Passed on to the constructor of `BuilderConfig`.
    """
    super(RobonetConfig, self).__init__(
        version=tfds.core.Version('4.0.1'), **kwargs)
    if (width is None) ^ (height is None):
      raise ValueError('Either both dimensions should be set, or none of them')
    self.sample_dataset = sample_dataset
    self.width = width
    self.height = height


class Robonet(tfds.core.BeamBasedBuilder):
  """RoboNet: Large-Scale Multi-Robot Learning."""

  BUILDER_CONFIGS = [
      RobonetConfig(
          name='robonet_sample_64',
          description='64x64 RoboNet Sample.',
          sample_dataset=True,
          width=64,
          height=64,
      ),
      RobonetConfig(
          name='robonet_sample_128',
          description='128x128 RoboNet Sample.',
          sample_dataset=True,
          width=128,
          height=128,
      ),
      RobonetConfig(
          name='robonet_64',
          description='64x64 RoboNet.',
          sample_dataset=False,
          width=64,
          height=64,
      ),
      RobonetConfig(
          name='robonet_128',
          description='128x128 RoboNet.',
          sample_dataset=False,
          width=128,
          height=128,
      ),
  ]

  def _info(self):
    if self.builder_config.width is not None:
      if self.builder_config.height is None:
        raise ValueError('Provide either both height and width or none.')
      ffmpeg_extra_args = (
          '-vf', 'scale={}x{}'.format(self.builder_config.height,
                                      self.builder_config.width))
    else:
      ffmpeg_extra_args = []

    video_shape = (
        None, self.builder_config.height, self.builder_config.width, 3)

    features = tfds.features.FeaturesDict({
        # Video frames: uint8 [None, Time, Width, Height, Channels]
        'video': tfds.features.Video(
            video_shape,
            ffmpeg_extra_args=ffmpeg_extra_args,
            encoding_format='png'),
        # Robot actions: float32, [None, ACTIONS_DIM]
        'actions': tfds.features.Tensor(
            shape=(None, ACTIONS_DIM), dtype=tf.float32),
        # Robot states: float32, [None, STATE_DIM]
        'states': tfds.features.Tensor(
            shape=(None, STATES_DIM), dtype=tf.float32),
        # Filename: Text
        'filename': tfds.features.Text()
    })

    return tfds.core.DatasetInfo(
        builder=self,
        description=textwrap.dedent("""\
        RoboNet contains over 15 million video frames of robot-object
        interaction, taken from 113 unique camera viewpoints.

        * The actions are deltas in position and rotation to the robot
        end-effector with one additional dimension of the action vector
        reserved for the gripper joint.

        * The states are cartesian end-effector control action space
        with restricted rotation, and a gripper joint"""),
        features=features,
        homepage='https://www.robonet.wiki/',
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    files = dl_manager.download_and_extract(
        DATA_URL_SAMPLE if self.builder_config.sample_dataset else DATA_URL)
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                'filedir': os.path.join(files, 'hdf5'),
            }),
    ]

  def _build_pcollection(self, pipeline, filedir):
    """Generate examples as dicts."""
    beam = tfds.core.lazy_imports.apache_beam

    def _process_example(filename):
      """Converts one video from hdf5 format."""
      h5py = tfds.core.lazy_imports.h5py
      with h5py.File(filename) as hf:
        video_bytes = hf['env']['cam0_video']['frames'][:].tostring()
        states = hf['env']['state'][:].astype(np.float32)
        states = np.pad(
            states, ((0, 0), (0, STATES_DIM-states.shape[1])), 'constant')
        actions = hf['policy']['actions'][:].astype(np.float32)
        actions = np.pad(
            actions, ((0, 0), (0, ACTIONS_DIM-actions.shape[1])), 'constant')

      basename = os.path.basename(filename)
      features = {
          'video': video_bytes,
          'actions': actions,
          'states': states,
          'filename': basename,
      }
      return basename, features

    filenames = tf.io.gfile.glob(os.path.join(filedir, '*.hdf5'))
    return (
        pipeline
        | beam.Create(filenames)
        | beam.Map(_process_example)
    )



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tensorflow_datasets.core.transform.image.image_transform."""

import os

import numpy as np
import tensorflow.compat.v2 as tf
from tensorflow_datasets import testing
from tensorflow_datasets.core import decode as decode_lib
from tensorflow_datasets.core import features as features_lib
from tensorflow_datasets.core import utils

tf.enable_v2_behavior()

randint = np.random.randint


class BaseDecodeTest(testing.FeatureExpectationsTestCase):

  def test_image_custom_decode(self):

    # Do not uses random here because Jpeg compression has loss, so decoded
    # value isn't the same
    img_shaped = np.ones(shape=(30, 60, 3), dtype=np.uint8)
    x, y, w, h = 4, 7, 10, 13
    img_cropped = img_shaped[y:y + h, x:x + w, :]

    class DecodeCrop(decode_lib.Decoder):
      """Simple class on how to customize the decoding."""

      def decode_example(self, serialized_image):
        return tf.image.decode_and_crop_jpeg(
            serialized_image,
            [y, x, h, w],
            channels=self.feature.shape[-1],
        )

    @decode_lib.make_decoder()
    def decode_crop(serialized_image, feature):
      return tf.image.decode_and_crop_jpeg(
          serialized_image,
          [y, x, h, w],
          channels=feature.shape[-1],
      )

    image_path = os.fspath(utils.tfds_path('testing/test_data/test_image.jpg'))
    with tf.io.gfile.GFile(image_path, 'rb') as f:
      serialized_img = f.read()

    self.assertFeature(
        # Image with statically defined shape
        feature=features_lib.Image(shape=(30, 60, 3), encoding_format='jpeg'),
        shape=(30, 60, 3),
        dtype=tf.uint8,
        tests=[
            testing.FeatureExpectationItem(
                value=img_shaped,
                expected=img_cropped,
                shape=(13, 10, 3),  # Shape is cropped
                decoders=DecodeCrop(),
            ),
            testing.FeatureExpectationItem(
                value=img_shaped,
                expected=img_cropped,
                shape=(13, 10, 3),  # Shape is cropped
                decoders=decode_crop(),  # pylint: disable=no-value-for-parameter
            ),
            testing.FeatureExpectationItem(
                value=image_path,
                expected=serialized_img,
                shape=(),
                dtype=tf.string,
                decoders=decode_lib.SkipDecoding(),
            ),
        ],
    )

  def test_video_custom_decode(self):

    image_path = os.fspath(utils.tfds_path('testing/test_data/test_image.jpg'))
    with tf.io.gfile.GFile(image_path, 'rb') as f:
      serialized_img = f.read()

    self.assertFeature(
        # Image with statically defined shape
        feature=features_lib.Video(shape=(None, 30, 60, 3)),
        shape=(None, 30, 60, 3),
        dtype=tf.uint8,
        tests=[
            testing.FeatureExpectationItem(
                value=[image_path] * 15,  # 15 frames of video
                expected=[serialized_img] * 15,  # Non-decoded image
                shape=(15,),
                dtype=tf.string,  # Only string are decoded
                decoders=decode_lib.SkipDecoding(),
            ),
        ],
    )

    # Test with FeatureDict
    self.assertFeature(
        feature=features_lib.FeaturesDict({
            'image': features_lib.Image(
                shape=(30, 60, 3), encoding_format='jpeg'),
            'label': tf.int64,
        }),
        shape={
            'image': (30, 60, 3),
            'label': (),
        },
        dtype={
            'image': tf.uint8,
            'label': tf.int64,
        },
        tests=[
            testing.FeatureExpectationItem(
                decoders={
                    'image': decode_lib.SkipDecoding(),
                },
                value={
                    'image': image_path,
                    'label': 123,
                },
                expected={
                    'image': serialized_img,
                    'label': 123,
                },
                shape={
                    'image': (),
                    'label': (),
                },
                dtype={
                    'image': tf.string,
                    'label': tf.int64,
                },
            ),
        ],
    )


if __name__ == '__main__':
  testing.test_main()



"""TODO(duke_ultranet): Add a description here."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import numpy as np
import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds
import logging

# TODO(duke_ultranet): BibTeX citation
_CITATION = """
"""

# TODO(duke_ultranet):
# Be sure to run pip install apache-beam==2.19.0
# and pip uninstall typing
_DESCRIPTION = {
    'channel': '''
        ```
        # Processing script
        
        DATASET_NAME=duke_ultranet/channel
        GCP_PROJECT=duke-ultrasound
        GCS_BUCKET=gs://duke-research-us
        
        echo "git+git://github.com/ouwen/datasets@tfds" > /tmp/beam_requirements.txt
        python3 -m tensorflow_datasets.scripts.download_and_prepare \
          --datasets=$DATASET_NAME \
          --data_dir=$GCS_BUCKET/tensorflow_datasets \
          --beam_pipeline_options=\
        "runner=DataflowRunner,project=$GCP_PROJECT,job_name=tfds-duke-ultranet,"\
        "staging_location=$GCS_BUCKET/binaries,temp_location=$GCS_BUCKET/temp,"\
        "requirements_file=/tmp/beam_requirements.txt,region=us-east1,"\
        "autoscaling_algorithm=NONE,num_workers=20,"\
        "machine_type=n1-highmem-16,experiments=shuffle_mode=service,disk_size_gb=500"
        ```
    
    ''',
    'iq_channel': '',
    'dynamic_rx_beamformed': '''
        ```
        # Processing script
        
        DATASET_NAME=duke_ultranet/dynamic_rx_beamformed
        GCP_PROJECT=duke-ultrasound
        GCS_BUCKET=gs://duke-research-us
        
        echo "git+git://github.com/ouwen/datasets@tfds" > /tmp/beam_requirements.txt
        python3 -m tensorflow_datasets.scripts.download_and_prepare \
          --datasets=$DATASET_NAME \
          --data_dir=$GCS_BUCKET/tensorflow_datasets \
          --beam_pipeline_options=\
        "runner=DataflowRunner,project=$GCP_PROJECT,job_name=tfds-duke-ultranet,"\
        "staging_location=$GCS_BUCKET/binaries,temp_location=$GCS_BUCKET/temp,"\
        "requirements_file=/tmp/beam_requirements.txt,region=us-east1,"\
        "autoscaling_algorithm=NONE,num_workers=20,"\
        "machine_type=n1-highmem-16,experiments=shuffle_mode=service,disk_size_gb=500"
        ```
    ''',
    'iq_dynamic_rx_beamformed': '',
    'b_mode': '''
        ```
        # Processing script
        
        DATASET_NAME=duke_ultranet/b_mode
        GCP_PROJECT=duke-ultrasound
        GCS_BUCKET=gs://duke-research-us
        
        echo "git+git://github.com/ouwen/datasets@tfds" > /tmp/beam_requirements.txt
        python3 -m tensorflow_datasets.scripts.download_and_prepare \
          --datasets=$DATASET_NAME \
          --data_dir=$GCS_BUCKET/tensorflow_datasets \
          --beam_pipeline_options=\
        "runner=DataflowRunner,project=$GCP_PROJECT,job_name=tfds-duke-ultranet-bmode,"\
        "staging_location=$q/binaries,temp_location=$GCS_BUCKET/temp,"\
        "requirements_file=/tmp/beam_requirements.txt,region=us-east1,"\
        "autoscaling_algorithm=NONE,num_workers=20,"\
        "machine_type=n1-standard-16,experiments=shuffle_mode=service,disk_size_gb=50"
        ```
    '''
}

_HOMEPAGE = """
"""

_BUCKET = 'gs://duke-research-us/hindexbooster'

_NAN_FILES = {75, 682, 3925, 2851, 3170, 1123, 2049, 45, 100, 37, 4, 5, 7, 4104, 2057, 4106, 4107, 2062, 4110, 4111, 4112, 4116, 4119, 25, 2074, 4122, 4124, 32, 2080, 4131, 2084, 4137, 2096, 4148, 54, 4152, 57, 4157, 4159, 2113, 67, 2115, 4166, 71, 72, 73, 2122, 2124, 4176, 2130, 2131, 85, 2135, 2136, 4183, 4185, 2140, 4193, 4197, 4199, 2152, 4210, 2169, 4217, 124, 2174, 4223, 4226, 4228, 2181, 4231, 4232, 137, 4234, 2187, 4236, 2190, 2193, 146, 2198, 2199, 4246, 2203, 158, 2207, 2209, 2213, 166, 2214, 170, 2220, 4269, 174, 2228, 2234, 187, 4284, 195, 2244, 201, 4299, 2252, 4300, 4301, 4307, 2264, 2267, 4319, 225, 228, 4329, 2282, 236, 4334, 2289, 243, 251, 4348, 253, 259, 265, 271, 272, 4367, 4369, 275, 4371, 2325, 4377, 288, 2338, 292, 4388, 309, 2357, 4405, 312, 315, 2364, 4415, 321, 2369, 2372, 2374, 2376, 4427, 4429, 337, 4441, 346, 4443, 4451, 4455, 2418, 4469, 2428, 4481, 2435, 388, 390, 4488, 4493, 4495, 4502, 408, 411, 420, 4516, 422, 4517, 424, 425, 4518, 4520, 428, 4521, 4523, 432, 4528, 2483, 436, 443, 444, 2491, 4543, 453, 2502, 2504, 2508, 4557, 462, 2511, 4560, 2515, 4568, 2522, 2525, 4573, 482, 488, 2536, 4585, 2539, 493, 494, 2545, 4596, 502, 514, 518, 2566, 2571, 4619, 4620, 4621, 2576, 532, 2587, 540, 542, 544, 548, 2597, 550, 552, 562, 2611, 4660, 2614, 2623, 4675, 2630, 585, 2634, 591, 2654, 609, 4706, 611, 2660, 4710, 4715, 620, 630, 639, 4735, 2695, 2699, 653, 2706, 2715, 4764, 2721, 691, 692, 701, 2750, 4801, 4802, 709, 713, 2761, 2762, 4810, 2765, 2767, 723, 4820, 725, 728, 4824, 2783, 738, 4835, 741, 746, 2796, 2800, 2801, 757, 2805, 759, 4854, 4859, 777, 2825, 2827, 2829, 2832, 785, 2841, 794, 2842, 4892, 2847, 4896, 801, 2849, 4904, 811, 812, 2862, 2863, 4913, 4917, 822, 824, 4920, 4922, 4927, 836, 2884, 4935, 840, 2892, 4942, 4946, 2901, 858, 863, 865, 866, 873, 4971, 876, 4974, 2929, 2932, 886, 4989, 894, 895, 4990, 904, 908, 2956, 911, 2963, 2971, 2972, 927, 2977, 2978, 936, 2991, 3005, 959, 3007, 3010, 965, 966, 968, 3016, 970, 3020, 3023, 3024, 977, 3032, 985, 3038, 991, 3044, 3046, 3057, 3063, 3065, 1019, 1022, 1024, 1026, 3080, 3082, 1038, 3086, 3089, 3094, 3100, 1053, 1054, 3101, 1057, 1058, 1063, 3116, 1070, 1071, 3118, 3119, 3120, 1076, 3128, 3144, 1100, 1102, 3153, 1106, 1108, 3157, 3158, 3159, 3165, 1120, 3169, 3174, 3176, 1130, 1131, 3183, 1150, 3200, 1153, 3205, 1161, 3211, 1166, 1172, 3229, 1182, 3242, 1199, 1210, 1215, 1219, 1220, 1222, 3271, 1241, 3291, 1244, 3293, 1252, 3301, 1261, 1262, 1263, 3309, 3316, 1270, 3345, 1300, 3353, 1310, 1315, 1316, 3364, 1319, 1322, 3373, 1326, 1330, 3380, 1334, 3385, 1339, 1340, 3388, 1355, 3403, 3405, 1361, 1367, 1369, 3417, 3422, 1375, 1378, 1385, 3442, 1400, 1405, 3457, 1414, 1417, 3465, 3471, 3490, 1443, 1445, 3500, 1463, 1464, 3515, 1472, 3523, 3524, 1481, 3534, 1490, 1496, 1501, 1502, 3551, 1517, 3568, 1540, 1542, 3592, 3610, 1569, 3617, 1576, 1577, 3630, 3631, 1585, 1596, 3644, 3648, 3652, 1611, 3661, 1614, 1619, 3667, 1622, 1631, 1635, 3690, 1643, 3691, 1645, 3695, 1648, 1652, 1657, 1659, 1670, 3721, 1676, 3724, 3725, 1682, 1689, 1690, 3737, 1694, 3747, 1702, 3750, 1708, 3766, 1719, 1720, 1729, 1730, 1731, 1732, 3783, 1736, 3785, 3786, 1744, 1746, 3797, 1751, 1753, 3801, 3803, 3805, 1758, 1759, 1769, 3823, 3833, 1786, 3841, 3849, 1814, 3870, 3873, 1826, 1827, 3878, 1834, 1845, 3897, 3899, 3906, 1859, 1865, 3916, 3918, 1884, 1886, 1888, 1893, 3955, 1909, 1911, 3959, 1914, 3964, 1917, 3974, 3980, 1936, 1939, 1944, 1948, 3999, 1957, 1958, 1963, 4012, 4015, 1968, 4016, 1974, 4029, 1982, 1983, 4030, 1987, 1995, 4046, 2003, 4051, 4053, 2008, 4058, 4059, 2014, 2019, 4071, 4075, 4079, 4085, 4093, 4095}

_BAD_FILES = {3584, 1025, 2048, 1538, 2564, 513, 4616, 3594, 3595, 1549, 4623, 2070, 3095, 2584, 3097, 23, 27, 1532, 1227, 3104, 4641, 1571, 3620, 4132, 39, 2601, 1066, 2090, 555, 1069, 2606, 47, 2607, 3117, 1074, 4657, 567, 2616, 569, 4670, 3135, 1090, 2628, 4677, 2631, 3663, 3668, 1621, 597, 3675, 4699, 1118, 3168, 98, 1634, 4194, 1125, 4200, 1641, 2668, 1647, 624, 3187, 2164, 1651, 1142, 4723, 4726, 3706, 3195, 4222, 127, 4737, 130, 1668, 3716, 645, 3717, 3210, 1164, 4238, 3216, 1171, 149, 3223, 3735, 3738, 4252, 2717, 4254, 4767, 159, 2210, 163, 1188, 1189, 4774, 3239, 4264, 4777, 171, 3243, 4268, 3757, 4274, 180, 182, 3767, 696, 694, 4790, 699, 3772, 2231, 191, 2752, 707, 2757, 198, 197, 714, 3787, 202, 205, 4814, 719, 1738, 1233, 4816, 3795, 1748, 213, 724, 3794, 217, 221, 4318, 1247, 3808, 1761, 4321, 4834, 4327, 3820, 3308, 751, 240, 3311, 4340, 3319, 4856, 1784, 1274, 4347, 1788, 3325, 4350, 1789, 1280, 1793, 2306, 3334, 1803, 267, 269, 2830, 3852, 1298, 3860, 4373, 3861, 3352, 4376, 3869, 4389, 806, 296, 297, 3884, 813, 3374, 4399, 2352, 820, 310, 3895, 823, 1337, 3386, 2363, 316, 2875, 831, 1346, 1347, 327, 3400, 842, 1354, 4426, 333, 3917, 2387, 4435, 853, 4438, 1879, 3928, 343, 4442, 2395, 4949, 855, 2398, 3530, 2916, 356, 3943, 4969, 4458, 4459, 3438, 2414, 367, 4465, 370, 1906, 4981, 1398, 1399, 2424, 2944, 897, 3459, 1923, 3975, 2952, 1931, 1935, 914, 3477, 2455, 3480, 3992, 2463, 1951, 4005, 427, 941, 944, 4530, 946, 1460, 439, 953, 3513, 1978, 1468, 3519, 4545, 4034, 1476, 3013, 1480, 2505, 2506, 971, 4556, 3532, 1486, 4559, 976, 2000, 4047, 3535, 981, 470, 3544, 4570, 3547, 4060, 3037, 3040, 481, 3043, 484, 4068, 2026, 491, 1516, 4590, 2544, 499, 2548, 1524, 1523, 2552, 508, 4094}

_FILES = list(set(range(5000)) - set(_NAN_FILES) - set(_BAD_FILES))

_TX_POS = np.array([-1.3188923611111137e-02, -1.2874774305555581e-02, -1.2560625000000027e-02, -1.2246475694444469e-02, -1.1932994791666688e-02, -1.1619513888888907e-02, -1.1305364583333352e-02, -1.0991215277777798e-02, -1.0677065972222240e-02, -1.0362916666666689e-02, -1.0048767361111127e-02, -9.7346180555555762e-03, -9.4204687500000217e-03, -9.1063194444444672e-03, -8.7928385416666897e-03, -8.4793576388889121e-03, -8.1652083333333559e-03, -7.8510590277778014e-03, -7.5369097222222452e-03, -7.2227604166666915e-03, -6.9086111111111318e-03, -6.5944618055555765e-03, -6.2803125000000211e-03, -5.9668315972222436e-03, -5.6533506944444660e-03, -5.3392013888889098e-03, -5.0250520833333553e-03, -4.7109027777777999e-03, -4.3967534722222455e-03, -4.0826041666666901e-03, -3.7684548611111308e-03, -3.4543055555555737e-03, -3.1401562500000210e-03, -2.8266753472222447e-03, -2.5125260416666881e-03, -2.1983767361111310e-03, -1.8842274305555758e-03, -1.5700781250000205e-03, -1.2559288194444655e-03, -9.4177951388891006e-04, -6.2763020833335394e-04, -3.1348090277779836e-04, -1.9931249937525191e-17,  3.1348090277775808e-04,  6.2763020833331339e-04,  9.4177951388886799e-04,  1.2559288194444239e-03,  1.5700781249999799e-03,  1.8842274305555346e-03,  2.1983767361110906e-03,  2.5125260416666473e-03,  2.8266753472222014e-03,  3.1401562499999793e-03,  3.4543055555555351e-03,  3.7684548611110909e-03,  4.0826041666666467e-03,  4.3967534722222030e-03,  4.7109027777777574e-03,  5.0250520833333119e-03,  5.3392013888888682e-03,  5.6533506944444235e-03,  5.9668315972222002e-03,  6.2803124999999778e-03,  6.5944618055555322e-03,  6.9086111111110876e-03,  7.2227604166666438e-03,  7.5369097222222009e-03,  7.8510590277777563e-03,  8.1652083333333108e-03,  8.4793576388888670e-03,  8.7928385416666446e-03,  9.1063194444444238e-03,  9.4204687499999801e-03,  9.7346180555555346e-03,  1.0048767361111091e-02,  1.0362916666666647e-02,  1.0677065972222203e-02,  1.0991215277777758e-02,  1.1305364583333312e-02,  1.1619513888888868e-02,  1.1932994791666646e-02,  1.2246475694444425e-02,  1.2560624999999983e-02,  1.2874774305555539e-02,  1.3188923611111096e-02])

_SUB_APERTURE_SIZE = 96
_CHANNELS = 180

class DukeUltranet(tfds.core.BeamBasedBuilder):
    """TODO(duke_ultranet): Short description of my dataset."""

    VERSION = tfds.core.Version('0.2.3') # adds downsampled baseband IQ, adds test set
    SUPPORTED_VERSIONS = [
          tfds.core.Version('0.2.2') # dynamic_rx and bmode are cached'
      ]
    BUILDER_CONFIGS = [
        tfds.core.BuilderConfig(
            version=VERSION,
            name="channel",
            description="Raw channel data from simulated transducers."
        ),
        tfds.core.BuilderConfig(
            version=VERSION,
            name="iq_channel",
            description="IQ Raw channel data from simulated transducers."
        ),
        tfds.core.BuilderConfig(
            version=VERSION,
            name="dynamic_rx_beamformed",
            description="Dynamic recieve beamformed data windowed to target of interest."
        ),
        tfds.core.BuilderConfig(
            version=VERSION,
            name="iq_dynamic_rx_beamformed",
            description="IQ Dynamic recieve beamformed data windowed to target of interest."
        ),
        tfds.core.BuilderConfig(
            version=VERSION,
            name="b_mode",
            description="B mode image."
        )
    ]

    @staticmethod
    def get_rf_hdf5(filepath, i):
        h5py = tfds.core.lazy_imports.h5py
        
        with h5py.File(tf.io.gfile.GFile(filepath, 'rb'), mode='r') as f:
            nowall = f.get('rf_nowall')[()]
            wall = f.get('rf_wall')[()]
        return {
            'nowall': nowall,
            'wall': wall,
            'i': i
        }

    @staticmethod
    @tf.function
    def tf_demodulate(x, fs, f0, axis=-1):
        '''Uses baseband IQ and decreases axial samples by factor of 10'''
        fs = tf.cast(fs, tf.complex64)
        f0 = tf.cast(f0, tf.complex64)
        initial_downsampling = 2
        iq = DukeUltranet.tf_hilbert(x[..., ::initial_downsampling], axis=-1)
        t = tf.cast(tf.range(0, tf.shape(iq)[-1]), tf.complex64)*1/(fs/initial_downsampling)
        iq = iq*tf.math.exp(-1j*2*np.pi*f0*t[None, None, :])
        return iq[..., ::5]

    @staticmethod
    @tf.function
    def tf_hilbert(x, axis=-1):
        '''Performs 1d hilbert similar to scipy'''

        # Change axes to be most inner for fft 
        axis = tf.constant(axis)
        if axis < 0: axis = tf.rank(x) + axis
        axes = tf.range(tf.rank(x))
        axes = tf.math.mod(axes - tf.reduce_max(axes) + axis, tf.size(axes))
        x = tf.transpose(x, perm=axes)

        # Apply fft
        x = tf.cast(x, tf.complex64)
        Xf = tf.signal.fft(x)

        # Create 2U 
        N = tf.shape(Xf)[-1]
        h = tf.cast(tf.ones([N//2 + 1])*2, Xf.dtype)
        if tf.math.mod(N,2) == 0: h = tf.tensor_scatter_nd_update(h, [[0],[tf.size(h)-1]], [1,1])
        else: h = tf.tensor_scatter_nd_update(h, [[0]], [1])
        h = tf.concat([h, tf.zeros(N-tf.size(h), dtype=h.dtype)], axis=0)

        # Apply ifft and hilbert
        x = tf.signal.ifft(Xf*h)

        # Change axes back
        x = tf.transpose(x, perm=tf.argsort(axes))
        return x

    @staticmethod
    @tf.function
    def tf_log10(x):
        return tf.math.log(x) / tf.math.log(tf.constant(10, dtype=x.dtype))

    @staticmethod
    @tf.function
    def tf_db(x):
        return tf.constant(20, dtype=x.dtype) * DukeUltranet.tf_log10(x)

    @staticmethod
    @tf.function
    def batch_interp1d(x, x_min, x_max, y):
        '''
        y is an 3d array of function to interpolate where the last dimension is to be interpolated [..., N]
        x is a 3d array equal of locations to eval and is monotonically increasing
        '''
        x_max = x_max - 1
        valid_x = (x > x_min) & (x < x_max)

        x = tf.clip_by_value(x, x_min, x_max)
        x_floor = tf.cast(tf.math.floor(x), tf.int32)
        x_ceil = tf.cast(tf.math.ceil(x), tf.int32)

        y_floor = tf.gather(y, x_floor, axis=-1, batch_dims=2)
        y_ceil = tf.gather(y, x_ceil, axis=-1, batch_dims=2)

        out = y_floor + (y_ceil - y_floor)*(x - tf.cast(x_floor, y.dtype))
        out = out * tf.cast(valid_x, y.dtype)
        return out
    
    @staticmethod    
    def apply_delays(delays, data):
        '''
        data is a 3D array of shape (transmits, channels, axial_samples)
        delays is a 3D array of shape (transmits, channels, axial_samples)
        returns delayed data
        '''
        return DukeUltranet.batch_interp1d(delays, 0, tf.cast(delays.shape[-1], delays.dtype), data)

    @staticmethod
    @tf.function
    def beamform_dynamic_rx(data_shape, tx_pos, rx_pos, s0=0, c=1540, fs=None, s0_var=None, data_dtype=tf.float64):
        '''
        beamform_dynamic_rx will beamform focused data with a dynamic rx delay profile

        data is a 3D array of shape (transmits, channels, axial_samples)
        tx_pos is a 2D array shape (tx_beam_num, 3) where the last channel is (x,y,z) coordinates in cartesian space of the start beam.
        rx_pos is a 2D array shape (rx_beam_num, 3) where the last channel is (x,y,z) coordiates in cartesian space of the end beam.
        s0 is the sample number offset globally
        s0_var is sample number of the first sample across transmits
        c is the speed of sound (m/s)
        fs is the sampling frequency (Hz)
        tx_delays is an array of time delays (s) used to focus the beam.
        '''
        transmits = data_shape[0]
        channels = data_shape[1]
        axial_samples = data_shape[2]
        if fs is None: raise ValueError('fs cannot be None')
        if s0_var is None: s0_var = tf.zeros(transmits, dtype=data_dtype)
        c = tf.cast(c, data_dtype)
        fs = tf.cast(fs, data_dtype)
        s0 = tf.cast(s0, data_dtype)
        tx_pos = tf.cast(tx_pos, data_dtype)
        rx_pos = tf.cast(rx_pos, data_dtype)
        tx_pos = tx_pos[:,None,:]
        rx_pos = rx_pos[None, :, :]
        dx = c/fs
        x1=(s0+1)*dx
        dz = tf.cast(tf.range(axial_samples, delta=1, dtype=tf.int32), data_dtype)/2*dx
        tx_beam_path = dz[None, None, ...]

        # TODO only works for linear beams, use law of cosines for calculating sectorscans
        distance_from_beam = tf.norm(tx_pos - rx_pos, axis=2)[...,None]
        rx_echo_path = tf.sqrt(tx_beam_path**2 + distance_from_beam**2) + s0_var[..., None, None]*dx

        total_distance = rx_echo_path+tx_beam_path-x1
        s = total_distance/dx
        return s

    def _info(self):
        common = {
            'c': tfds.features.Tensor(shape=(), dtype=tf.float64),
            'fs': tfds.features.Tensor(shape=(), dtype=tf.float64),
            'tiny_imagenet': {
                'target_number': tfds.features.Tensor(shape=(), dtype=tf.int64),
                'body_wall_number': tfds.features.Tensor(shape=(), dtype=tf.int64),
                'target': tfds.features.Image(shape=(64,64,3), encoding_format='jpeg'),
                'clutter_wall': tfds.features.Image(shape=(64,64,3), encoding_format='jpeg')
            },
            'probe': {
                'tx_delays': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS), dtype=tf.float64),
                'element_on_mask': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS), dtype=tf.bool),
                'element_positions': tfds.features.Tensor(shape=(_CHANNELS,3), dtype=tf.float64),
                'rx_positions': tfds.features.Tensor(shape=(_CHANNELS,3), dtype=tf.float64),
                'tx_positions': tfds.features.Tensor(shape=(len(_TX_POS),3), dtype=tf.float64),
                'impulse': tfds.features.Tensor(shape=(57,), dtype=tf.float64),
                'pulse': tfds.features.Tensor(shape=(97,), dtype=tf.float64),
                'excitation': tfds.features.Tensor(shape=(41,), dtype=tf.float64),
                'a': tfds.features.Tensor(shape=(5,), dtype=tf.float64),
                'b': tfds.features.Tensor(shape=(5,), dtype=tf.float64),
                'pitch': tfds.features.Tensor(shape=(), dtype=tf.float64),
                't0': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'f0_hz': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'focus_depth': tfds.features.Tensor(shape=(), dtype=tf.float64)
            },
            'sim': {
                'B': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'cmap': tfds.features.Tensor(shape=(1247, 1091), dtype=tf.float64),
                'atten': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'cfl': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'ncycles': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'omega0': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'p0': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'ppw': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'rho': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'td': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'v': tfds.features.Tensor(shape=(), dtype=tf.float64),
                'grid': {
                    'dT': tfds.features.Tensor(shape=(), dtype=tf.float64),
                    'dY': tfds.features.Tensor(shape=(), dtype=tf.float64),
                    'dZ': tfds.features.Tensor(shape=(), dtype=tf.float64),
                    'nT': tfds.features.Tensor(shape=(), dtype=tf.float64),
                    'nY': tfds.features.Tensor(shape=(), dtype=tf.float64),
                    'nZ': tfds.features.Tensor(shape=(), dtype=tf.float64),
                    'wY': tfds.features.Tensor(shape=(), dtype=tf.float64),
                    'wZ': tfds.features.Tensor(shape=(), dtype=tf.float64)
                },
            }
        }

        if self.builder_config.name is 'channel':
            return tfds.core.DatasetInfo(
                builder=self,
                description=_DESCRIPTION['channel'],
                features=tfds.features.FeaturesDict({
                    'data': {
                        'without_wall': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,7012), dtype=tf.float32),
                        'with_wall': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,7012), dtype=tf.float32)
                    },
                    'params': common
                }),
                supervised_keys=('data/with_wall', 'data/without_wall'),
                homepage=_HOMEPAGE,
                citation=_CITATION
            )
        
        if self.builder_config.name is 'iq_channel':
            return tfds.core.DatasetInfo(
                builder=self,
                description=_DESCRIPTION['iq_channel'],
                features=tfds.features.FeaturesDict({
                    'data': {
                        'without_wall_real': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,702), dtype=tf.float16),
                        'without_wall_imag': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,702), dtype=tf.float16),
                        'with_wall_real': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,702), dtype=tf.float16),
                        'with_wall_imag': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,702), dtype=tf.float16)
                    },
                    'params': common
                }),
                homepage=_HOMEPAGE,
                citation=_CITATION
            )
        
        if self.builder_config.name is 'dynamic_rx_beamformed':
            return tfds.core.DatasetInfo(
                builder=self,
                description=_DESCRIPTION['dynamic_rx_beamformed'],
                features=tfds.features.FeaturesDict({
                    'data': {
                        'without_wall': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,3117), dtype=tf.float32),
                        'with_wall': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,3117), dtype=tf.float32)
                    },
                    'params': common
                }),
                supervised_keys=('data/with_wall', 'data/without_wall'),
                homepage=_HOMEPAGE,
                citation=_CITATION
            )

        if self.builder_config.name is 'iq_dynamic_rx_beamformed':
            return tfds.core.DatasetInfo(
                builder=self,
                description=_DESCRIPTION['iq_dynamic_rx_beamformed'],
                features=tfds.features.FeaturesDict({
                    'data': {
                        'without_wall_real': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,312), dtype=tf.float16),
                        'without_wall_imag': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,312), dtype=tf.float16),
                        'with_wall_real': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,312), dtype=tf.float16),
                        'with_wall_imag': tfds.features.Tensor(shape=(len(_TX_POS),_CHANNELS,312), dtype=tf.float16)
                    },
                    'params': common
                }),
                homepage=_HOMEPAGE,
                citation=_CITATION
            )

        if self.builder_config.name is 'b_mode':
            return tfds.core.DatasetInfo(
                builder=self,
                description=_DESCRIPTION['b_mode'],
                features=tfds.features.FeaturesDict({
                    'data': {
                        'without_wall': tfds.features.Tensor(shape=(len(_TX_POS),3117), dtype=tf.float16),
                        'with_wall': tfds.features.Tensor(shape=(len(_TX_POS),3117), dtype=tf.float16),
                    },
                    'params': common
                }),
                supervised_keys=('data/with_wall', 'data/without_wall'),
                homepage=_HOMEPAGE,
                citation=_CITATION
            )

    def _split_generators(self, _):
        """Returns SplitGenerators."""
        return [
            tfds.core.SplitGenerator(
                    name=tfds.Split.TRAIN,
                    gen_kwargs={
                        'files': _FILES[1200:]
                    }
            ),
            tfds.core.SplitGenerator(
                    name=tfds.Split.TEST,
                    gen_kwargs={
                        'files': _FILES[:1200]
                    }
            )
        ]

    def _build_pcollection(self, pipeline, files):
        beam = tfds.core.lazy_imports.apache_beam
        h5py = tfds.core.lazy_imports.h5py
        signal = tfds.core.lazy_imports.scipy.signal
        
        filepaths_rf = []
        for file_num in files:
            for i in range(1,len(_TX_POS)+1):
                filepaths_rf.append((file_num, '{}/results/val_{}/rf_{}.mat'.format(_BUCKET, file_num, str(i).zfill(2)), i))

        def _download_rf(job):
            file_num, filepath, i = tuple(job)
            data = None
            try:
                data = DukeUltranet.get_rf_hdf5(filepath, i)
            except Exception as e:
                logging.error(e)
                logging.error(filepath)
                beam.metrics.Metrics.counter('results', "download-error").inc()
                return
            beam.metrics.Metrics.counter('results', "rf-downloaded").inc()
            yield file_num, data

        def _check_size(job):
            file_num, rf = job
            rf = list(rf)
            return len(rf) == 85

        def _process(job):
            file_num, rf = tuple(job)

            metadata_filepath = '{}/results/val_{}/structs_42.mat'.format(_BUCKET, file_num)
            target_jpeg_filepath = '{}/images/val_{}.JPEG'.format(_BUCKET, file_num+5000)
            body_jpeg_filepath = '{}/images/val_{}.JPEG'.format(_BUCKET, file_num)

            m = h5py.File(tf.io.gfile.GFile(metadata_filepath, 'rb'), mode='r')
            elements = m.get('/xdc/out')[()].shape[-1]
            on_elements = [np.arange(i,_SUB_APERTURE_SIZE+i, dtype=np.int64) for i in range(len(_TX_POS))]
            tx_delays = np.zeros((len(_TX_POS), elements))
            for i, on_ele in enumerate(on_elements): tx_delays[i, on_ele] = np.squeeze(m.get('xdc/delays'))[()]

            on_bool = np.zeros((len(_TX_POS), elements))
            for i, on_ele in enumerate(on_elements): on_bool[i, on_ele] = np.ones(_SUB_APERTURE_SIZE)
            on_bool = on_bool.astype(bool)

            common = {
                'c': np.squeeze(m.get('acq_params/c')[()]),
                'fs': np.squeeze(m.get('acq_params/fs')[()]),
                'tiny_imagenet': {
                    'target_number': file_num+5000,
                    'body_wall_number': file_num,
                    'target': target_jpeg_filepath,
                    'clutter_wall': body_jpeg_filepath
                },
                'probe': {
                    'tx_delays': tx_delays,
                    'element_on_mask': on_bool,
                    'element_positions': np.transpose(m.get('xdc/out')[()]),
                    'rx_positions': np.transpose(m.get('xdc/out')[()]),
                    'tx_positions': np.transpose(np.stack([_TX_POS, np.zeros_like(_TX_POS), np.zeros_like(_TX_POS)])),
                    'impulse': np.squeeze(m.get('xdc/impulse')[()]),
                    'pulse': np.squeeze(m.get('xdc/pulse')[()]),
                    'excitation': np.squeeze(m.get('xdc/excitation')[()]),
                    'a': np.squeeze(m.get('bf_params/a')[()]),
                    'b': np.squeeze(m.get('bf_params/b')[()]),
                    'pitch': np.squeeze(m.get('xdc/pitch')[()]),
                    't0': np.squeeze(m.get('xdc/t0')[()]),
                    'f0_hz': np.squeeze(m.get('acq_params/f0')[()]),
                    'focus_depth': np.squeeze(m.get('xdc/focus')[()][2])
                },
                'sim': {
                    'B': np.squeeze(m.get('input_vars/B')[()]),
                    'cmap': np.squeeze(m.get('field_maps/cmap')[()]),
                    'atten': np.squeeze(m.get('input_vars/atten')[()]),
                    'cfl': np.squeeze(m.get('input_vars/cfl')[()]),
                    'ncycles': np.squeeze(m.get('input_vars/ncycles')[()]),
                    'omega0': np.squeeze(m.get('input_vars/omega0')[()]),
                    'p0': np.squeeze(m.get('input_vars/p0')[()]),
                    'ppw': np.squeeze(m.get('input_vars/ppw')[()]),
                    'rho': np.squeeze(m.get('input_vars/rho')[()]),
                    'td': np.squeeze(m.get('input_vars/td')[()]),
                    'v': np.squeeze(m.get('input_vars/v')[()]),
                    'grid': {
                        'dT': np.squeeze(m.get('grid_vars/dT')[()]),
                        'dY': np.squeeze(m.get('grid_vars/dY')[()]),
                        'dZ': np.squeeze(m.get('grid_vars/dZ')[()]),
                        'nT': np.squeeze(m.get('grid_vars/nT')[()]),
                        'nY': np.squeeze(m.get('grid_vars/nY')[()]),
                        'nZ': np.squeeze(m.get('grid_vars/nZ')[()]),
                        'wY': np.squeeze(m.get('input_vars/wY')[()]),
                        'wZ': np.squeeze(m.get('input_vars/wZ')[()]),
                    }
                }
            }

            a = common['probe']['a'].astype(np.float32)
            b = common['probe']['b'].astype(np.float32)
            fs = common['fs'].astype(np.float32)
            f0 = common['probe']['f0_hz'].astype(np.float32)

            rf = list(rf)
            ordering = np.array([r['i'] for r in rf])
            ordering = list(np.argsort(ordering))
            nowall = []
            wall = []
            for idx in ordering:
                nowall.append(rf[idx]['nowall'])
                wall.append(rf[idx]['wall'])
            nowall = np.stack(nowall)
            wall = np.stack(wall)
            rf = None
            nowall = signal.lfilter(b, a, nowall, axis=-1)
            wall = signal.lfilter(b, a, wall, axis=-1)

            if self.builder_config.name is 'channel':
                varying = {
                    'without_wall': nowall,
                    'with_wall': wall
                }

            if self.builder_config.name is 'iq_channel':
                nowall = self.tf_demodulate(nowall, fs, f0, axis=-1)
                wall = self.tf_demodulate(wall, fs, f0, axis=-1)

                varying = {
                    'without_wall_real': tf.cast(tf.math.real(nowall), tf.float16),
                    'without_wall_imag': tf.cast(tf.math.imag(nowall), tf.float16),
                    'with_wall_real': tf.cast(tf.math.real(wall), tf.float16),
                    'with_wall_imag': tf.cast(tf.math.imag(wall), tf.float16)
                }

            if self.builder_config.name is 'dynamic_rx_beamformed' or self.builder_config.name is 'b_mode' or self.builder_config.name is 'iq_dynamic_rx_beamformed':
                s = self.beamform_dynamic_rx(nowall.shape, 
                                             common['probe']['tx_positions'], 
                                             common['probe']['rx_positions'], 
                                             s0=common['probe']['t0']*common['fs'], 
                                             fs=common['fs'], data_dtype=tf.float64)
                s = tf.cast(s, tf.float32)
                nowall = self.apply_delays(s, nowall)[..., 3117:6234]
                wall = self.apply_delays(s, wall)[..., 3117:6234]

                if self.builder_config.name is 'iq_dynamic_rx_beamformed':
                    nowall = self.tf_demodulate(nowall, fs, f0, axis=-1)
                    wall = self.tf_demodulate(wall, fs, f0, axis=-1)

                varying = {
                    'without_wall_real': tf.cast(tf.math.real(nowall), tf.float16),
                    'without_wall_imag': tf.cast(tf.math.imag(nowall), tf.float16),
                    'with_wall_real': tf.cast(tf.math.real(wall), tf.float16),
                    'with_wall_imag': tf.cast(tf.math.imag(wall), tf.float16)
                }

            if self.builder_config.name is 'b_mode':
                on_mask = tf.cast(common['probe']['element_on_mask'][..., None], tf.float32)
                env = tf.abs(self.tf_hilbert(tf.reduce_sum(nowall*on_mask, axis=1), axis=-1))
                env = env/tf.reduce_max(env)
                nowall = self.tf_db(env)

                env = tf.abs(self.tf_hilbert(tf.reduce_sum(wall*on_mask, axis=1), axis=-1))
                env = env/tf.reduce_max(env)
                wall = self.tf_db(env)

                varying = {
                    'without_wall': tf.cast(nowall, tf.float16),
                    'with_wall': tf.cast(wall, tf.float16)
                }
            m.close()

            beam.metrics.Metrics.counter('results', "rf-processed").inc()
            yield file_num, {
                'data': varying,
                'params': common
            }

        return (
            pipeline
            | beam.Create(filepaths_rf)
            | beam.FlatMap(_download_rf)
            | beam.GroupByKey()
            | beam.Filter(_check_size)
            | beam.FlatMap(_process)
        )



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Stable hashing function using md5.

Note that the properties we are looking at here are:
 1- Good distribution of hashes (random uniformity);
 2- Speed;
 3- Availability as portable library, giving the same hash independently of
 platform.

Crypto level hashing is not a requirement.

A bit of history:
 - CityHash was first used. However the C implementation used complex
 instructions and was hard to compile on some platforms.
 - Siphash was chosen as a replacement, because although being slower,
 it has a simpler implementation and it has a pure Python implementation, making
 it easier to distribute TFDS on Windows or MAC. However, the used library
 (reference C implementation wrapped using cffi) crashed the python interpreter
 on py3 with tf1.13.
 - So md5, although being slower that the two above works everywhere and is
 still faster than a pure python implementation of siphash.

Changing the hash function should be done thoughfully, as it would change the
order of datasets (and thus sets of records when using slicing API). If done,
all datasets would need to have their major version bumped.

Note that if we were to find a dataset for which two different keys give the
same hash (collision), a solution could be to append the key to its hash.

The split name is being used as salt to avoid having the same keys in two splits
result in same order.
"""

import hashlib
from typing import Union

import six
import tensorflow.compat.v2 as tf


HashKey = Union[str, bytes, int]


def _to_bytes(data):
  if not isinstance(data, (six.string_types, bytes)):
    data = str(data)
  elif isinstance(data, str):
    # For windows compatibility, we normalize the key in case a
    # filepath is passed as key ('path\\to\\file' -> 'path/to/file')
    data = data.replace('\\', '/')
  return tf.compat.as_bytes(data)


class Hasher(object):
  """Hasher: to initialize a md5 with salt."""

  def __init__(self, salt):
    self._md5 = hashlib.md5(_to_bytes(salt))

  def hash_key(self, key: HashKey) -> int:
    """Returns 128 bits hash of given key.

    Args:
      key (bytes, string or anything convertible to a string): key to be hashed.
        If the key is a string, it will be encoded to bytes using utf-8.
        If the key is neither a string nor bytes, it will be converted to a str,
          then to bytes.
        This means that `"1"` (str) and `1` (int) will have the same hash. The
        intent of the hash being to shuffle keys, it is recommended that all
        keys of a given set to shuffle use a single type.

    Returns:
      128 bits integer, hash of key.
    """
    md5 = self._md5.copy()
    md5.update(_to_bytes(key))
    return int(md5.hexdigest(), 16)



"""jsrt dataset."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import tensorflow_datasets.public_api as tfds
import tensorflow as tf
import numpy as np
import os

_CITATION = """
@MISC{Shiraishi2000-kf,
    title   = "Development of a Digital Image Database for Chest Radiographs With
                         and Without a Lung Nodule",
    author  = "Shiraishi, Junji and Katsuragawa, Shigehiko and Ikezoe, Junpei and
                         Matsumoto, Tsuneo and Kobayashi, Takeshi and Komatsu, Ken-Ichi and
                         Matsui, Mitate and Fujita, Hiroshi and Kodera, Yoshie and Doi,
                         Kunio",
    journal = "American Journal of Roentgenology",
    volume  =  174,
    number  =  1,
    pages   = "71--74",
    year    =  2000
}
"""

_DESCRIPTION = """JSRT is a high resoultion (2048x2048, 0.175mm pixel) chest 
x-ray imaging dataset with 4096 gray scale (12bit). It contains 154 images with
a nodule, and 93 non-nodule images."""

_DIFFICULTY = ['Extremely Subtle', 'Very Subtle', 'Subtle', 'Relatively Obvious', 'Obvious']

class Jsrt(tfds.core.GeneratorBasedBuilder):
    VERSION = tfds.core.Version('0.1.0')
    MANUAL_DOWNLOAD_INSTRUCTIONS = """\
        manual_dir should contain unzipped All247Images.zip and Clinical_Information.zip from the JSRT Database
    """

    @staticmethod
    def hist_match(source, template):
        oldshape = source.shape
        source = source.ravel()
        template = template.ravel()
        s_values, bin_idx, s_counts = np.unique(source, return_inverse=True,
                                                                                        return_counts=True)
        t_values, t_counts = np.unique(template, return_counts=True)
        s_quantiles = np.cumsum(s_counts).astype(np.float64)
        s_quantiles /= s_quantiles[-1]
        t_quantiles = np.cumsum(t_counts).astype(np.float64)
        t_quantiles /= t_quantiles[-1]
        interp_t_values = np.interp(s_quantiles, t_quantiles, t_values)
        return interp_t_values[bin_idx].reshape(oldshape)

    def _info(self):
        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict({
                'image': tfds.features.Image(shape=(2048, 2048, 1), encoding_format='png', dtype=tf.uint16),
                'bse': tfds.features.Image(shape=(2048, 2048, 1), encoding_format='png', dtype=tf.uint16),
                'age': tfds.features.Tensor(shape=(), dtype=tf.int8),
                'sex': tfds.features.ClassLabel(names=['male', 'female', 'other']),
                'nodule': tfds.features.ClassLabel(names=['non-nodule', 'nodule']),
                'nodule_details': {
                    'difficulty': tfds.features.ClassLabel(names=_DIFFICULTY),
                    'size_mm': tfds.features.Tensor(shape=(), dtype=tf.uint8),
                    'x_coord': tfds.features.Tensor(shape=(), dtype=tf.uint16),
                    'y_coord': tfds.features.Tensor(shape=(), dtype=tf.uint16),
                    'type': tfds.features.ClassLabel(names=['benign', 'malignant']),
                    'diagnosis': tfds.features.Tensor(shape=(), dtype=tf.string)
                }
            }),
            homepage='https://www.jsrt.or.jp/data/',
            citation=_CITATION
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        lung_nodule_list = os.path.join(dl_manager.manual_dir, 'CLNDAT_EN.txt')
        no_nodule_list = os.path.join(dl_manager.manual_dir, 'CNNDAT_EN.txt')
        jsrt_directory = os.path.join(dl_manager.manual_dir, 'JSRT')
        bse_directory = os.path.join(dl_manager.manual_dir, 'BSE')
        if not (tf.io.gfile.exists(lung_nodule_list) or 
                        tf.io.gfile.exists(no_nodule_list) or
                        tf.io.gfile.exists(jsrt_directory) or
                        tf.io.gfile.exists(bse_directory)):
            msg = "You must download the dataset files manually and unzip them in: {}, {}, {}, and {}".format(lung_nodule_list, no_nodule_list, jsrt_directory, bse_directory)
            raise AssertionError(msg)

        with tf.io.gfile.GFile(no_nodule_list, "r") as f:
            lines = f.read().split('\r')
        nn = []
        for line in lines:
            line = line.split(' ')
            line = list(filter(lambda x: x is not '', line))
            nn.append(line)
        nn = nn[:-1]

        with tf.io.gfile.GFile(lung_nodule_list, "r") as f:
            lines = f.read().split('\r')
        ln = [line.split('\t') for line in lines]
        ln = ln[:-2]
        return [
            tfds.core.SplitGenerator(
                    name=tfds.Split.TRAIN,
                    gen_kwargs={
                        "lung_nodule_list": ln,
                        "no_nodule_list": nn,
                        "jsrt_directory": jsrt_directory,
                        "bse_directory": bse_directory
                    }
            )
        ]

    def _generate_examples(self, lung_nodule_list=None, no_nodule_list=None, jsrt_directory=None, bse_directory=None):
        """Yields examples."""
        def get_images(filename):
            image = tf.io.decode_raw(tf.io.read_file('{}/{}.IMG'.format(jsrt_directory, filename)), tf.uint16, little_endian=False)
            image = tf.reshape(image, (2048, 2048, 1))
            bse = tf.io.decode_png(tf.io.read_file('{}/{}.png'.format(bse_directory, filename)), channels=1, dtype=tf.uint16)
            bse = self.hist_match(bse.numpy(), image.numpy()).astype(np.uint16)
            return image.numpy(), bse

        for line in no_nodule_list:
            filename = line[0][:-4]
            image, bse = get_images(filename)
            yield filename, {
                'image': image,
                'bse': bse,
                'age': int(line[1]),
                'sex': line[2],
                'nodule': 'non-nodule',
                'nodule_details': {
                    'difficulty': _DIFFICULTY[0],
                    'size_mm': 0,
                    'x_coord': 0,
                    'y_coord': 0,
                    'type': 'benign',
                    'diagnosis': ""
                }
            }

        for line in lung_nodule_list:
            filename = line[0][:-4]
            image, bse = get_images(filename)
            try:
                age = int(line[3])
            except Exception:
                age = -1

            yield filename, {
                'image': image,
                'bse': bse,
                'age': age,
                'sex': line[4],
                'nodule': 'nodule',
                'nodule_details': {
                    'difficulty': _DIFFICULTY[int(line[1])-1],
                    'size_mm': int(line[2]),
                    'x_coord': int(line[5]),
                    'y_coord': int(line[6]),
                    'type': line[7],
                    'diagnosis': " ".join(line[8:]),
                }
            }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""TensorFlow compatibility utilities."""

# pylint: disable=g-import-not-at-top,g-direct-tensorflow-import

import contextlib
import distutils.version
import functools
import os

MIN_TF_VERSION = "2.1.0"

_ensure_tf_install_called = False


# Ensure TensorFlow is importable and its version is sufficiently recent. This
# needs to happen before anything else, since the imports below will try to
# import tensorflow, too.
def ensure_tf_install():  # pylint: disable=g-statement-before-imports
  """Attempt to import tensorflow, and ensure its version is sufficient.

  Raises:
    ImportError: if either tensorflow is not importable or its version is
    inadequate.
  """
  # Only check the first time.
  global _ensure_tf_install_called
  if _ensure_tf_install_called:
    return
  _ensure_tf_install_called = True

  try:
    import tensorflow.compat.v2 as tf  # pylint: disable=import-outside-toplevel
  except ImportError:
    # Print more informative error message, then reraise.
    print("\n\nFailed to import TensorFlow. Please note that TensorFlow is not "
          "installed by default when you install TensorFlow Datasets. This is "
          "so that users can decide whether to install the GPU-enabled "
          "TensorFlow package. To use TensorFlow Datasets, please install the "
          "most recent version of TensorFlow, by following instructions at "
          "https://tensorflow.org/install.\n\n")
    raise

  tf_version = distutils.version.LooseVersion(tf.__version__)
  min_tf_version = distutils.version.LooseVersion(MIN_TF_VERSION)
  if tf_version < min_tf_version:
    raise ImportError(
        "This version of TensorFlow Datasets requires TensorFlow "
        f"version >= {MIN_TF_VERSION}; Detected an installation of version "
        f"{tf.__version__}. Please upgrade TensorFlow to proceed."
    )


def is_dataset(ds):
  """Whether ds is a Dataset. Compatible across TF versions."""
  import tensorflow.compat.v2 as tf  # pylint: disable=import-outside-toplevel
  return isinstance(ds, (tf.data.Dataset, tf.compat.v1.data.Dataset))


def _make_pathlike_fn(fn, nb_path_arg=1):
  """Wrap the function in a PathLike-compatible function."""

  @functools.wraps(fn)
  def new_fn(*args, **kwargs):
    # Normalize PathLike objects
    args = tuple(os.fspath(arg) for arg in args[:nb_path_arg])
    return fn(*args, **kwargs)

  return new_fn


@contextlib.contextmanager
def mock_gfile_pathlike():
  """Contextmanager which patch the `tf.io.gfile` API to be PathLike compatible.

  Before TF 2.4, GFile API is not PathLike compatible.
  After TF2.4, this function is a no-op.

  Yields:
    None
  """
  import tensorflow.compat.v2 as tf  # pylint: disable=import-outside-toplevel
  import tensorflow_datasets.testing as tfds_test  # pytype: disable=import-error

  class GFile(tf.io.gfile.GFile):

    def __init__(self, fpath, *args, **kwargs):
      super().__init__(os.fspath(fpath), *args, **kwargs)

  tf_version = distutils.version.LooseVersion(tf.__version__)
  min_tf_version = distutils.version.LooseVersion("2.4.0")
  if tf_version >= min_tf_version:
    yield  # No-op for recent TF versions
  else:  # Legacy TF, patch TF to restore PathLike compatibility
    with contextlib.ExitStack() as stack:
      for fn_name, nb_path_arg in [
          ("copy", 2),  # Use str, as tf.io.gfile.copy.__name__ == 'copy_v2'
          ("exists", 1),
          ("glob", 1),
          ("isdir", 1),
          ("listdir", 1),
          ("makedirs", 1),
          ("mkdir", 1),
          ("remove", 1),
          ("rename", 2),
          ("rmtree", 1),
          ("stat", 1),
          ("walk", 1),
      ]:
        fn = getattr(tf.io.gfile, fn_name)
        new_fn = _make_pathlike_fn(fn, nb_path_arg)
        stack.enter_context(tfds_test.mock_tf(f"tf.io.gfile.{fn_name}", new_fn))
      stack.enter_context(tfds_test.mock_tf("tf.io.gfile.GFile", GFile))
      yield



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

r"""Tool to generate the dataset catalog documentation.
"""

import argparse
import os
from typing import List, Optional

from absl import app
from absl.flags import argparse_flags
import dataclasses

import tensorflow as tf
import tensorflow_datasets as tfds
from tensorflow_datasets.scripts.documentation import document_datasets
import yaml


def _parse_flags(_) -> argparse.Namespace:
  """Command line flags."""
  parser = argparse_flags.ArgumentParser(
      prog='build_catalog',
      description='Tool to generate the dataset catalog documentation',
  )
  parser.add_argument(
      '--datasets',
      help='Comma separated list of datasets to document. None for all '
      'datasets.',
  )
  parser.add_argument(
      '--catalog_dir',
      help='Directory path where to generate the catalog. Default to TFDS dir.',
  )
  return parser.parse_args()


def main(args: argparse.Namespace):
  catalog_dir = args.catalog_dir or os.path.join(
      tfds.core.utils.tfds_write_path(),
      'docs',
      'catalog',
  )

  build_catalog(
      datasets=args.datasets.split(',') if args.datasets else None,
      catalog_dir=catalog_dir,
  )


@dataclasses.dataclass
class DatasetItem(object):
  name: str
  path: str
  is_nightly: bool = False


def create_section_toc(section, section_datasets):
  heading = '\n### `%s`\n' % section
  nightly_suffix = ' ' + document_datasets.NightlyDocUtil.icon
  entries = [
      f' * [`{item.name}`]({item.path})' +
      (item.is_nightly and nightly_suffix or '') for item in section_datasets
  ]
  return '\n'.join([heading] + entries)


def build_catalog(
    datasets: Optional[List[str]] = None,
    catalog_dir: Optional[str] = None,
    toc_relative_path: str = '/datasets/catalog/',
) -> None:
  """Document all datasets, including the table of content.

  Args:
    datasets: Lists of dataset to document (all if not set)
    catalog_dir: Destination path for the catalog
    toc_relative_path: Relative path of the catalog directory, used to
      generate the table of content relative links.
  """
  # Build datasets doc
  print('Build datasets overview...')
  overview_doc, datasets_dict = document_datasets.dataset_docs_str(datasets)

  # For _toc.yaml
  toc_dictionary = {'toc': [{
      'title': 'Overview',
      'path': os.path.join(toc_relative_path, 'overview'),
  }]}

  section_tocs = []

  nightly_util = document_datasets.NightlyDocUtil()

  print('Build Sections')
  for section, datasets_in_section in sorted(list(datasets_dict.items())):
    print('Section %s...' % section)
    section_str = section.replace('_', ' ').capitalize()
    sec_dict = {'title': section_str}
    sec_paths = list()
    section_toc = []
    for dataset_name, is_manual, doc in datasets_in_section:
      print('Dataset %s...' % dataset_name)

      sidebar_item = {
          'path': os.path.join(toc_relative_path, dataset_name),
          'title': dataset_name + (' (manual)' if is_manual else '')
      }
      ds_item = DatasetItem(
          name=dataset_name,
          path=dataset_name + '.md',
      )
      if nightly_util.is_builder_nightly(dataset_name):
        sidebar_item['status'] = 'nightly'
        ds_item.is_nightly = True

      sec_paths.append(sidebar_item)
      section_toc.append(ds_item)

      dataset_file = os.path.join(catalog_dir, dataset_name + '.md')
      with tf.io.gfile.GFile(dataset_file, 'w') as f:
        f.write(doc)

    section_tocs.append(create_section_toc(section_str, section_toc))
    sec_dict['section'] = sec_paths
    toc_dictionary['toc'].append(sec_dict)

  with tf.io.gfile.GFile(os.path.join(catalog_dir, 'overview.md'), 'w') as f:
    f.write(overview_doc.format(toc='\n'.join(section_tocs)))

  with tf.io.gfile.GFile(os.path.join(catalog_dir, '_toc.yaml'), 'w') as f:
    yaml.dump(toc_dictionary, f, default_flow_style=False)


if __name__ == '__main__':
  app.run(main, flags_parser=_parse_flags)



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for open_images dataset module."""
from tensorflow_datasets import testing
from tensorflow_datasets.object_detection import open_images


class OpenImagesV42012Test(testing.DatasetBuilderTestCase):
  DATASET_CLASS = open_images.OpenImagesV4
  SPLITS = {  # Expected number of examples on each split.
      'train': 512,
      'test': 36,
      'validation': 12,
  }
  DL_EXTRACT_RESULT = {
      'train_images': ['s3-tar_train_sha1_%s.tar' % i
                       for i in '0123456789abcdef'],
      'test_images': 's3-tar_test_sha2.tar',
      'validation_images': 's3-tar_validation_sha3.tar',
      'train_human_labels': 'train-human-labels.csv',
      'train_machine_labels': 'train-machine-labels.csv',
      'test_human_labels': 'test-human-labels.csv',
      'test_machine_labels': 'test-machine-labels.csv',
      'validation_human_labels': 'validation-human-labels.csv',
      'validation_machine_labels': 'validation-machine-labels.csv',
      'train-annotations-bbox': 'train-annotations-bbox.csv',
      'test-annotations-bbox': 'test-annotations-bbox.csv',
      'validation-annotations-bbox': 'validation-annotations-bbox.csv',
  }


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""generates api_docs for tensorflow_datasets."""
import os

from absl import app
from absl import flags

import tensorflow_datasets as tfds
from tensorflow_datasets import testing

from tensorflow_docs.api_generator import generate_lib

FLAGS = flags.FLAGS

flags.DEFINE_string("output_dir", "/tmp/datasets_api",
                    "Where to output the docs")
flags.DEFINE_string(
    "code_url_prefix",
    "https://github.com/tensorflow/datasets/tree/master/tensorflow_datasets/",
    "The url prefix for links to code.")

flags.DEFINE_bool("search_hints", True,
                  "Include metadata search hints in the generated files")

flags.DEFINE_string("site_path", "datasets/api_docs/python",
                    "Path prefix in the _toc.yaml")


def execute(output_dir, code_url_prefix, search_hints, site_path):
  """Builds API docs for tensorflow_datasets."""
  # Internally, tfds.testing defaults to None. Fill it in here so that we get
  # documentation.
  tfds.testing = testing
  doc_generator = generate_lib.DocGenerator(
      root_title="TensorFlow Datasets",
      py_modules=[("tfds", tfds)],
      base_dir=os.path.dirname(tfds.__file__),
      search_hints=search_hints,
      code_url_prefix=code_url_prefix,
      site_path=site_path)

  doc_generator.build(output_dir)


def main(unused_argv):
  execute(FLAGS.output_dir, FLAGS.code_url_prefix, FLAGS.search_hints,
          FLAGS.site_path)


if __name__ == "__main__":
  app.run(main)



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test for PetFinder."""

from tensorflow_datasets import testing
from tensorflow_datasets.image_classification import pet_finder


class PetFinderTest(testing.DatasetBuilderTestCase):
  # petfinder:
  DATASET_CLASS = pet_finder.PetFinder
  SPLITS = {
      'train': 2,  # Number of fake train example
      'test': 2,  # Number of fake test example
  }
  DL_EXTRACT_RESULT = {
      'train': 'train.csv',
      'train_images': 'train_images',
      'test': 'test.csv',
      'test_images': 'test_images',
  }


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test for CoS-E dataset module."""

from tensorflow_datasets import testing
from tensorflow_datasets.text import cos_e


class CosETest(testing.DatasetBuilderTestCase):
  DATASET_CLASS = cos_e.CosE
  SPLITS = {
      "validation": 2,  # Number of fake validation examples
      "train": 2,  # Number of fake train examples
  }

  DL_EXTRACT_RESULT = {
      "dev": ["cos_e_dev.jsonl"],
      "train": ["cos_e_train.jsonl"],
      "cqa_dev": "cqa_dev.jsonl",
      "cqa_train": "cqa_train.jsonl",
      "cqa_test": "cqa_test.jsonl",
  }


if __name__ == "__main__":
  testing.test_main()



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ClassLabel feature."""

import os
import six
import tensorflow.compat.v2 as tf
from tensorflow_datasets.core.features import feature
from tensorflow_datasets.core.utils import type_utils

Json = type_utils.Json


class ClassLabel(feature.Tensor):
  """`FeatureConnector` for integer class labels."""

  def __init__(self, *, num_classes=None, names=None, names_file=None):
    """Constructs a ClassLabel FeatureConnector.

    There are 3 ways to define a ClassLabel, which correspond to the 3
    arguments:

     * `num_classes`: create 0 to (num_classes-1) labels
     * `names`: a list of label strings
     * `names_file`: a file containing the list of labels.

    Note: On python2, the strings are encoded as utf-8.

    Args:
      num_classes: `int`, number of classes. All labels must be < num_classes.
      names: `list<str>`, string names for the integer classes. The
        order in which the names are provided is kept.
      names_file: `str`, path to a file with names for the integer
        classes, one per line.
    """
    super(ClassLabel, self).__init__(shape=(), dtype=tf.int64)

    self._num_classes = None
    self._str2int = None
    self._int2str = None

    # The label is explicitly set as undefined (no label defined)
    if all(a is None for a in (num_classes, names, names_file)):
      return

    if sum(a is not None for a in (num_classes, names, names_file)) != 1:
      raise ValueError(
          "Only a single argument of ClassLabel() should be provided.")

    if num_classes is not None:
      self._num_classes = num_classes
    elif names is not None:
      self.names = names
    else:
      self.names = _load_names_from_file(names_file)

  @property
  def num_classes(self):
    return self._num_classes

  @property
  def names(self):
    if not self._int2str:
      return [tf.compat.as_text(str(i)) for i in range(self._num_classes)]
    return list(self._int2str)

  @names.setter
  def names(self, new_names):
    int2str = [tf.compat.as_text(name) for name in new_names]
    # Names can only be defined once
    if self._int2str is not None and self._int2str != int2str:
      raise ValueError(
          "Trying to overwrite already defined ClassLabel names. Previous: {} "
          ", new: {}".format(self._int2str, int2str))

    # Set-up [new] names
    self._int2str = int2str
    self._str2int = {name: i for i, name in enumerate(self._int2str)}
    if len(self._int2str) != len(self._str2int):
      raise ValueError(
          "Some label names are duplicated. Each label name should be unique.")

    # If num_classes has been defined, ensure that num_classes and names match
    num_classes = len(self._str2int)
    if self._num_classes is None:
      self._num_classes = num_classes
    elif self._num_classes != num_classes:
      raise ValueError(
          "ClassLabel number of names do not match the defined num_classes. "
          "Got {} names VS {} num_classes".format(
              num_classes, self._num_classes)
      )

  def str2int(self, str_value):
    """Conversion class name string => integer."""
    str_value = tf.compat.as_text(str_value)
    if self._str2int:
      return self._str2int[str_value]

    # No names provided, try to integerize
    failed_parse = False
    try:
      int_value = int(str_value)
    except ValueError:
      failed_parse = True
    if failed_parse or not 0 <= int_value < self._num_classes:
      raise ValueError("Invalid string class label %s" % str_value)
    return int_value

  def int2str(self, int_value):
    """Conversion integer => class name string."""
    if self._int2str:
      # Maybe should support batched np array/eager tensors, to allow things
      # like
      # out_ids = model(inputs)
      # labels = cifar10.info.features['label'].int2str(out_ids)
      return self._int2str[int_value]

    # No names provided, return str(int)
    if not 0 <= int_value < self._num_classes:
      raise ValueError("Invalid integer class label %d" % int_value)
    return tf.compat.as_text(str(int_value))

  def encode_example(self, example_data):
    if self._num_classes is None:
      raise ValueError(
          "Trying to use ClassLabel feature with undefined number of class. "
          "Please set ClassLabel.names or num_classes."
      )

    # If a string is given, convert to associated integer
    if isinstance(example_data, six.string_types):
      example_data = self.str2int(example_data)

    # Allowing -1 to mean no label.
    if not -1 <= example_data < self._num_classes:
      raise ValueError("Class label %d greater than configured num_classes %d" %
                       (example_data, self._num_classes))
    return example_data

  def save_metadata(self, data_dir, feature_name=None):
    """See base class for details."""
    # Save names if defined
    if self._str2int is not None:
      names_filepath = _get_names_filepath(data_dir, feature_name)
      _write_names_to_file(names_filepath, self.names)

  def load_metadata(self, data_dir, feature_name=None):
    """See base class for details."""
    # Restore names if defined
    names_filepath = _get_names_filepath(data_dir, feature_name)
    if tf.io.gfile.exists(names_filepath):
      self.names = _load_names_from_file(names_filepath)

  def _additional_repr_info(self):
    return {"num_classes": self.num_classes}

  def repr_html(self, ex: int) -> str:
    """Class labels are displayed with their name."""
    if ex == -1:
      return "-"
    elif not self._int2str:
      return str(ex)
    else:
      return f"{ex} ({self.int2str(ex)})"

  @classmethod
  def from_json_content(cls, value: Json) -> "ClassLabel":
    return cls(**value)

  def to_json_content(self) -> Json:
    return {"num_classes": self.num_classes}


def _get_names_filepath(data_dir, feature_name):
  return os.path.join(data_dir, "{}.labels.txt".format(feature_name))


def _load_names_from_file(names_filepath):
  names_filepath = os.fspath(names_filepath)
  with tf.io.gfile.GFile(names_filepath, "r") as f:
    return [
        name.strip()
        for name in tf.compat.as_text(f.read()).split("\n")
        if name.strip()  # Filter empty names
    ]


def _write_names_to_file(names_filepath, names):
  with tf.io.gfile.GFile(names_filepath, "w") as f:
    f.write("\n".join(names) + "\n")



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""e2e_cleaned dataset."""

import csv

import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_CITATION = r"""
@inproceedings{dusek-etal-2019-semantic,
    title = "Semantic Noise Matters for Neural Natural Language Generation",
    author = "Du{\v{s}}ek, Ond{\v{r}}ej  and
      Howcroft, David M.  and
      Rieser, Verena",
    booktitle = "Proceedings of the 12th International Conference on Natural Language Generation",
    month = oct # "{--}" # nov,
    year = "2019",
    address = "Tokyo, Japan",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/W19-8652",
    doi = "10.18653/v1/W19-8652",
    pages = "421--426",
    abstract = "Neural natural language generation (NNLG) systems are known for their pathological outputs, i.e. generating text which is unrelated to the input specification. In this paper, we show the impact of semantic noise on state-of-the-art NNLG models which implement different semantic control mechanisms. We find that cleaned data can improve semantic correctness by up to 97{\%}, while maintaining fluency. We also find that the most common error is omitting information, rather than hallucination.",
}
"""

_DESCRIPTION = """
An update release of E2E NLG Challenge data with cleaned MRs. The E2E data
contains dialogue act-based meaning representation (MR) in the restaurant domain
and up to 5 references in natural language, which is what one needs to predict.
"""

_HOMEPAGE_URL = 'https://github.com/tuetschek/e2e-cleaning'

_TRAIN_URL = 'https://github.com/tuetschek/e2e-cleaning/raw/master/cleaned-data/train-fixed.no-ol.csv'
_DEV_URL = 'https://github.com/tuetschek/e2e-cleaning/raw/master/cleaned-data/devel-fixed.no-ol.csv'
_TEST_URL = 'https://github.com/tuetschek/e2e-cleaning/raw/master/cleaned-data/test-fixed.csv'


def _get_table_from_mr(mr):
  """Converts a meaningful representation from e2e_cleaned dataset in a table."""
  mr_as_table = []
  for type_value in mr.split(', '):
    type_value_delimiter = type_value.find('[')
    type_ = type_value[0:type_value_delimiter]
    value = type_value[type_value_delimiter + 1:-1]
    mr_as_table.append({
        'column_header': type_,
        'row_number': 1,
        'content': value,
    })
  return mr_as_table


class E2eCleaned(tfds.core.GeneratorBasedBuilder):
  """MR in the restaurant domain and target utterances describing it."""

  VERSION = tfds.core.Version('0.1.0')

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        # This is the description that will appear on the datasets page.
        description=_DESCRIPTION,
        # tfds.features.FeatureConnectors
        features=tfds.features.FeaturesDict({
            'input_text': {
                'table':
                    tfds.features.Sequence({
                        'column_header': tf.string,
                        'row_number': tf.int16,
                        'content': tf.string,
                    })
            },
            'target_text': tf.string,
        }),
        # If there's a common (input, target) tuple from the features,
        # specify them here. They'll be used if as_supervised=True in
        # builder.as_dataset.
        supervised_keys=('input_text', 'target_text'),
        # Homepage of the dataset for documentation
        homepage=_HOMEPAGE_URL,
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    extracted_path = dl_manager.download_and_extract({
        'train_path': _TRAIN_URL,
        'dev_path': _DEV_URL,
        'test_path': _TEST_URL
    })
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            # These kwargs will be passed to _generate_examples
            gen_kwargs={'csv_path': extracted_path['train_path']},
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            # These kwargs will be passed to _generate_examples
            gen_kwargs={'csv_path': extracted_path['dev_path']},
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            # These kwargs will be passed to _generate_examples
            gen_kwargs={'csv_path': extracted_path['test_path']},
        ),
    ]

  def _generate_examples(self, csv_path):
    """Yields examples."""
    with tf.io.gfile.GFile(csv_path) as f:
      reader = csv.DictReader(f)
      for i, row in enumerate(reader):
        yield i, {
            'input_text': {
                'table': _get_table_from_mr(row['mr']),
            },
            'target_text': row['ref']
        }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""TensorFlow dataset for Binary Alphadigits."""
import six.moves.urllib as urllib
import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_URL = 'https://cs.nyu.edu/~roweis/data/'

_DESCRIPTION = ("Binary 20x16 digits of '0' through '9' and capital 'A' "
                "through 'Z'. 39 examples of each class.")

_IMAGE_SHAPE = (20, 16, 1)

_NAMES = [
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E',
    'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
    'U', 'V', 'W', 'X', 'Y', 'Z'
]

_CITATION = """\

"""


class BinaryAlphaDigits(tfds.core.GeneratorBasedBuilder):
  """Binary alphadigits dataset."""

  VERSION = tfds.core.Version('1.0.0')

  def _info(self):
    """Define the Dataset info."""

    return tfds.core.DatasetInfo(
        builder=self,
        description=(_DESCRIPTION),
        features=tfds.features.FeaturesDict({
            'image': tfds.features.Image(shape=_IMAGE_SHAPE),
            'label': tfds.features.ClassLabel(names=_NAMES),
        }),
        supervised_keys=('image', 'label'),
        homepage=_URL,
        citation=_CITATION)

  def _split_generators(self, dl_manager):
    """Define Splits for training data."""

    path = dl_manager.download(
        {'train': urllib.parse.urljoin(_URL, 'binaryalphadigs.mat')})

    return [
        tfds.core.SplitGenerator(
            name='train',
            gen_kwargs={
                'data_dir_path': path['train'],
            },
        )
    ]

  def _generate_examples(self, data_dir_path):
    """Generate Splits for training data."""

    with tf.io.gfile.GFile(data_dir_path, 'rb') as f:
      mat = tfds.core.lazy_imports.scipy.io.loadmat(f)
    for i in range(len(mat['dat'])):
      label = mat['classlabels'][0][i].item()
      for j in range(len(mat['dat'][i])):
        image = mat['dat'][i][j].reshape(20, 16, 1)
        yield '%d_%d'%(i, j), {'label': label, 'image': image}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tiny Shakespeare dataset."""

import os

import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_CITATION = """\
@misc{
  author={Karpathy, Andrej},
  title={char-rnn},
  year={2015},
  howpublished={\\url{https://github.com/karpathy/char-rnn}}
}"""

_DESCRIPTION = """\
40,000 lines of Shakespeare from a variety of Shakespeare's plays. \
Featured in Andrej Karpathy's blog post 'The Unreasonable Effectiveness of \
Recurrent Neural Networks': \
http://karpathy.github.io/2015/05/21/rnn-effectiveness/.

To use for e.g. character modelling:

```
d = tfds.load(name='tiny_shakespeare')['train']
d = d.map(lambda x: tf.strings.unicode_split(x['text'], 'UTF-8'))
# train split includes vocabulary for other splits
vocabulary = sorted(set(next(iter(d)).numpy()))
d = d.map(lambda x: {'cur_char': x[:-1], 'next_char': x[1:]})
d = d.unbatch()
seq_len = 100
batch_size = 2
d = d.batch(seq_len)
d = d.batch(batch_size)
```
"""


class TinyShakespeare(tfds.core.GeneratorBasedBuilder):
  """Tiny Shakespeare dataset builder."""

  VERSION = tfds.core.Version('1.0.0')

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({'text': tfds.features.Text()}),
        supervised_keys=None,
        homepage='https://github.com/karpathy/char-rnn/blob/master/data/tinyshakespeare/input.txt',
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    download_path = dl_manager.download(
        'https://raw.githubusercontent.com/karpathy/char-rnn/master/data/tinyshakespeare/input.txt'
    )
    if tf.io.gfile.isdir(download_path):
      # During testing the download manager mock gives us a directory
      txt_path = os.path.join(download_path, 'input.txt')
    else:
      txt_path = download_path
    with tf.io.gfile.GFile(txt_path, 'r') as f:
      text = f.read()

    # 90/5/5 split
    i = int(len(text) * 0.9)
    train_text, text = text[:i], text[i:]
    i = int(len(text) * 0.5)
    validation_text, text = text[:i], text[i:]
    test_text = text

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            # These kwargs will be passed to _generate_examples
            gen_kwargs={
                'split_key': 'train',
                'split_text': train_text
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={
                'split_key': 'validation',
                'split_text': validation_text
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                'split_key': 'test',
                'split_text': test_text
            },
        ),
    ]

  def _generate_examples(self, split_key, split_text):
    """Yields examples."""
    data_key = split_key  # Should uniquely identify the thing yielded
    feature_dict = {'text': split_text}
    yield data_key, feature_dict



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""BillSum Dataset."""

import json
import os

import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_CITATION = """
@misc{kornilova2019billsum,
    title={BillSum: A Corpus for Automatic Summarization of US Legislation},
    author={Anastassia Kornilova and Vlad Eidelman},
    year={2019},
    eprint={1910.00523},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DESCRIPTION = """
BillSum, summarization of US Congressional and California state bills.

There are several features:
  - text: bill text.
  - summary: summary of the bills.
  - title: title of the bills.
features for us bills. ca bills does not have.
  - text_len: number of chars in text.
  - sum_len: number of chars in summary.
"""

_URL = "https://drive.google.com/uc?export=download&id=1g89WgFHMRbr4QrvA0ngh26PY081Nv3lx"

_DOCUMENT = "text"
_SUMMARY = "summary"


class Billsum(tfds.core.GeneratorBasedBuilder):
  """BillSum Dataset."""

  # 2.0.0 data source updated to filter near duplicates.
  # 3.0.0  none of the test examples are 'near duplicates' of an example in the
  #   train set AND they dont have the same title, regardless of similarity.
  VERSION = tfds.core.Version("3.0.0")

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            _DOCUMENT: tfds.features.Text(),
            _SUMMARY: tfds.features.Text(),
            "title": tfds.features.Text(),
        }),
        supervised_keys=(_DOCUMENT, _SUMMARY),
        homepage="https://github.com/FiscalNote/BillSum",
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    dl_path = dl_manager.download_and_extract(_URL)
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                "path":
                    os.path.join(dl_path, "us_train_data_final_OFFICIAL.jsonl"),
                "key":
                    "bill_id"
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                "path":
                    os.path.join(dl_path, "us_test_data_final_OFFICIAL.jsonl"),
                "key":
                    "bill_id"
            },
        ),
        tfds.core.SplitGenerator(
            name="ca_test",
            gen_kwargs={
                "path":
                    os.path.join(dl_path, "ca_test_data_final_OFFICIAL.jsonl"),
                "key":
                    "external_id"
            },
        ),
    ]

  def _generate_examples(self, path=None, key=None):
    """Yields examples."""
    with tf.io.gfile.GFile(path) as f:
      for line in f:
        # in us bills, json has fields:
        #   text, summary, title, bill_id, text_len, sum_len
        # in ca bills, json has fields:
        #   text, summary, title, external_id
        d = json.loads(line)
        yield d[key], {k: d[k] for k in [_DOCUMENT, _SUMMARY, "title"]}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Opinosis Opinion Dataset."""

import os

import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_CITATION = """
@inproceedings{ganesan2010opinosis,
  title={Opinosis: a graph-based approach to abstractive summarization of highly redundant opinions},
  author={Ganesan, Kavita and Zhai, ChengXiang and Han, Jiawei},
  booktitle={Proceedings of the 23rd International Conference on Computational Linguistics},
  pages={340--348},
  year={2010},
  organization={Association for Computational Linguistics}
}
"""

_DESCRIPTION = """
The Opinosis Opinion Dataset consists of sentences extracted from reviews for 51 topics.
Topics and opinions are obtained from Tripadvisor, Edmunds.com and Amazon.com.
"""

_URL = "https://github.com/kavgan/opinosis-summarization/raw/master/OpinosisDataset1.0_0.zip"

_REVIEW_SENTS = "review_sents"
_SUMMARIES = "summaries"


class Opinosis(tfds.core.GeneratorBasedBuilder):
  """Opinosis Opinion Dataset."""

  VERSION = tfds.core.Version("1.0.0")

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            _REVIEW_SENTS: tfds.features.Text(),
            _SUMMARIES: tfds.features.Sequence(tfds.features.Text())
        }),
        supervised_keys=(_REVIEW_SENTS, _SUMMARIES),
        homepage="http://kavita-ganesan.com/opinosis/",
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    extract_path = dl_manager.download_and_extract(_URL)
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={"path": extract_path},
        ),
    ]

  def _generate_examples(self, path=None):
    """Yields examples."""
    topics_path = os.path.join(path, "topics")
    filenames = tf.io.gfile.listdir(topics_path)
    for filename in filenames:
      file_path = os.path.join(topics_path, filename)
      topic_name = filename.split(".txt")[0]
      with tf.io.gfile.GFile(file_path, "rb") as src_f:
        input_data = src_f.read()
      summaries_path = os.path.join(path, "summaries-gold", topic_name)
      summary_lst = []
      for summ_filename in sorted(tf.io.gfile.listdir(summaries_path)):
        file_path = os.path.join(summaries_path, summ_filename)
        with tf.io.gfile.GFile(file_path, "rb") as tgt_f:
          data = tgt_f.read().strip()
          summary_lst.append(data)
      summary_data = summary_lst
      yield filename, {_REVIEW_SENTS: input_data, _SUMMARIES: summary_data}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Titanic dataset."""

import collections
import csv
import numpy as np
import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_CITATION = """\
@ONLINE {titanic,
author = "Frank E. Harrell Jr., Thomas Cason",
title  = "Titanic dataset",
month  = "oct",
year   = "2017",
url    = "https://www.openml.org/d/40945"
}
"""

_DESCRIPTION = ("Dataset describing the survival status of "
                "individual passengers on the Titanic. Missing values in "
                "the original dataset are represented using ?. "
                "Float and int missing values are replaced with -1, string "
                "missing values are replaced with 'Unknown'.")

_EMBARKED_DICT = collections.OrderedDict([
    ("C", "Cherbourg"), ("Q", "Queenstown"), ("S", "Southampton"),
    ("?", "Unknown")
])

_PCLASS_DICT = collections.OrderedDict([
    ("1", "1st_class"), ("2", "2nd_class"), ("3", "3rd_class")
])

_SURVIVED_DICT = {"1": "survived", "0": "died"}


def convert_to_float(d):
  return -1.0 if d == "?" else np.float32(d)


def convert_to_int(d):
  return -1 if d == "?" else np.int32(d)


def convert_to_string(d):
  return "Unknown" if d == "?" else d


def convert_to_label(d, dictionary):
  return dictionary[d]


def return_same(d):
  return d


FEATURE_DICT = collections.OrderedDict([
    ("pclass", (tfds.features.ClassLabel(names=_PCLASS_DICT.values()),
                lambda d: convert_to_label(d, _PCLASS_DICT))),
    ("name", (tf.string, convert_to_string)),
    ("sex", (tfds.features.ClassLabel(names=["male", "female"]), return_same)),
    ("age", (tf.float32, convert_to_float)),
    ("sibsp", (tf.int32, convert_to_int)),
    ("parch", (tf.int32, convert_to_int)),
    ("ticket", (tf.string, convert_to_string)),
    ("fare", (tf.float32, convert_to_float)),
    ("cabin", (tf.string, convert_to_string)),
    ("embarked", (tfds.features.ClassLabel(names=_EMBARKED_DICT.values()),
                  lambda d: convert_to_label(d, _EMBARKED_DICT))),
    ("boat", (tf.string, convert_to_string)),
    ("body", (tf.int32, convert_to_int)),
    ("home.dest", (tf.string, convert_to_string))
])

_URL = "https://www.openml.org/data/get_csv/16826755/phpMYEkMl"


class Titanic(tfds.core.GeneratorBasedBuilder):
  """Titanic dataset."""
  VERSION = tfds.core.Version("2.0.0")
  RELEASE_NOTES = {
      "2.0.0": "New split API (https://tensorflow.org/datasets/splits)",
  }

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            "survived": tfds.features.ClassLabel(names=["died", "survived"]),
            "features": {name: dtype
                         for name, (dtype, func) in FEATURE_DICT.items()}
        }),
        supervised_keys=("features", "survived"),
        homepage="https://www.openml.org/d/40945",
        citation=_CITATION
        )

  def _split_generators(self, dl_manager):
    path = dl_manager.download(_URL)

    # There is no predefined train/val/test split for this dataset.
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                "file_path": path
            }),
    ]

  def _generate_examples(self, file_path):
    """Generate features and target given the directory path.

    Args:
      file_path: path where the csv file is stored

    Yields:
      The features and the target
    """

    with tf.io.gfile.GFile(file_path) as f:
      raw_data = csv.DictReader(f)
      for i, row in enumerate(raw_data):
        survive_val = row.pop("survived")
        yield i, {
            "survived": convert_to_label(survive_val, _SURVIVED_DICT),
            "features": {
                name: FEATURE_DICT[name][1](value)
                for name, value in row.items()
            }
        }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test for the PlantLeaves dataset."""

from tensorflow_datasets import testing
from tensorflow_datasets.image_classification import plantae_k


class PlantaeKTest(testing.DatasetBuilderTestCase):
  DATASET_CLASS = plantae_k.PlantaeK
  # See note below about the +1
  SPLITS = {"train": 16 + 1}
  _LABEL_TAGS = [
      "apple_d", "apple_h", "apricot_d", "apricot_h", "cherry_d", "cherry_h",
      "cranberry_d", "cranberry_h", "grapes_d", "grapes_h", "peach_d",
      "peach_h", "pear_d", "pear_h", "walnut_d", "walnut_h", "walnut-h"
  ]
  # NOTE: Must match file names in the test directory. Due to bug in file naming
  # we have to have both walnut_d and walnut_h for healthy walnut.
  DL_EXTRACT_RESULT = {
      fname: fname
      for fname in ["{}1.JPG".format(label_tag) for label_tag in _LABEL_TAGS]
  }


if __name__ == "__main__":
  testing.test_main()



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""`tfds.download.DownloadManager` API."""

from tensorflow_datasets.core.download.checksums import add_checksums_dir
from tensorflow_datasets.core.download.download_manager import DownloadConfig
from tensorflow_datasets.core.download.download_manager import DownloadManager
from tensorflow_datasets.core.download.downloader import DownloadError
from tensorflow_datasets.core.download.extractor import iter_archive
from tensorflow_datasets.core.download.resource import ExtractMethod
from tensorflow_datasets.core.download.resource import Resource
from tensorflow_datasets.core.download.util import ComputeStatsMode
from tensorflow_datasets.core.download.util import GenerateMode

__all__ = [
    "add_checksums_dir",
    "DownloadConfig",
    "DownloadManager",
    "DownloadError",
    "ComputeStatsMode",
    "GenerateMode",
    "Resource",
    "ExtractMethod",
    "iter_archive",
]



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Berkeley (BAIR) robot pushing dataset.

Self-Supervised Visual Planning with Temporal Skip Connections
Frederik Ebert, Chelsea Finn, Alex X. Lee, and Sergey Levine.
https://arxiv.org/abs/1710.05268
"""

import os

from absl import logging
import numpy as np
import tensorflow.compat.v2 as tf

import tensorflow_datasets.public_api as tfds

DATA_URL = "http://rail.eecs.berkeley.edu/datasets/bair_robot_pushing_dataset_v0.tar"

# There are exactly 30 frames in each video.
FRAMES_PER_VIDEO = 30
IMG_SHAPE = (64, 64, 3)


_CITATION = """\
@misc{1710.05268,
  Author = {Frederik Ebert and Chelsea Finn and Alex X. Lee and Sergey Levine},
  Title = {Self-Supervised Visual Planning with Temporal Skip Connections},
  Year = {2017},
  Eprint = {arXiv:1710.05268},
}
"""


class BairRobotPushingSmall(tfds.core.GeneratorBasedBuilder):
  """Robot pushing dataset from BAIR (Small 64x64 version)."""

  VERSION = tfds.core.Version("2.0.0")
  RELEASE_NOTES = {
      "2.0.0": "New split API (https://tensorflow.org/datasets/splits)",
  }

  def _info(self):
    # The Bair dataset consist of a sequence of frames (video) with associated
    # metadata (action and position)
    features = tfds.features.Sequence({
        "image_main": tfds.features.Image(shape=IMG_SHAPE),
        "image_aux1": tfds.features.Image(shape=IMG_SHAPE),
        "action": tfds.features.Tensor(shape=(4,), dtype=tf.float32),
        "endeffector_pos": tfds.features.Tensor(shape=(3,), dtype=tf.float32),
    }, length=FRAMES_PER_VIDEO)

    return tfds.core.DatasetInfo(
        builder=self,
        description="This data set contains roughly 44,000 examples of robot "
        "pushing motions, including one training set (train) and "
        "two test sets of previously seen (testseen) and unseen "
        "(testnovel) objects. This is the small 64x64 version.",
        features=features,
        homepage="https://sites.google.com/view/sna-visual-mpc/",
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    files = dl_manager.download_and_extract(DATA_URL)
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                "filedir": os.path.join(files, "softmotion30_44k", "train"),
            }),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                "filedir": os.path.join(files, "softmotion30_44k", "test"),
            }),
    ]

  def _generate_examples(self, filedir):
    logging.info("Reading data from %s.", filedir)
    files = tf.io.gfile.listdir(filedir)
    logging.info("%d files found.", len(files))

    # For each file
    for filename in sorted(tf.io.gfile.listdir(filedir)):
      filepath = os.path.join(filedir, filename)

      # For each video inside the file
      for video_id, example_str in enumerate(
          tf.compat.v1.io.tf_record_iterator(filepath)):
        example = tf.train.SequenceExample.FromString(example_str)

        # Merge all frames together
        all_frames = []
        for frame_id in range(FRAMES_PER_VIDEO):
          # Extract all features from the original proto context field
          frame_feature = {   # pylint: disable=g-complex-comprehension
              out_key: example.context.feature[in_key.format(frame_id)]   # pylint: disable=g-complex-comprehension
              for out_key, in_key in [
                  ("image_main", "{}/image_main/encoded"),
                  ("image_aux1", "{}/image_aux1/encoded"),
                  ("endeffector_pos", "{}/endeffector_pos"),
                  ("action", "{}/action"),
              ]
          }

          # Decode float
          for key in ("endeffector_pos", "action"):
            values = frame_feature[key].float_list.value
            frame_feature[key] = [values[i] for i in range(len(values))]

          # Decode images (from encoded string)
          for key in ("image_main", "image_aux1"):
            img = frame_feature[key].bytes_list.value[0]  # pytype: disable=attribute-error
            img = np.frombuffer(img, dtype=np.uint8)
            img = np.reshape(img, IMG_SHAPE)
            frame_feature[key] = img

          all_frames.append(frame_feature)

        # Encode the sequence (list) of frames (feature dicts)
        # yield [
        #     {'action': [...], 'image_main': img_frame0, ...},  # Frame 0
        #     {'action': [...], 'image_main': img_frame1, ...},  # Frame 1
        #     ...,
        # ]
        yield "%s_%s" % (filepath, video_id), all_frames



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""TED talk high/low-resource paired language data set from Qi, et al. 2018."""

import os

import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_DESCRIPTION = """\
Data sets derived from TED talk transcripts for comparing similar language pairs
where one is high resource and the other is low resource.
"""

_CITATION = """\
@inproceedings{Ye2018WordEmbeddings,
  author  = {Ye, Qi and Devendra, Sachan and Matthieu, Felix and Sarguna, Padmanabhan and Graham, Neubig},
  title   = {When and Why are pre-trained word embeddings useful for Neural Machine Translation},
  booktitle = {HLT-NAACL},
  year    = {2018},
  }
"""

_DATA_URL = "http://www.phontron.com/data/qi18naacl-dataset.tar.gz"

_VALID_LANGUAGE_PAIRS = (
    ("az", "en"),
    ("az_tr", "en"),
    ("be", "en"),
    ("be_ru", "en"),
    ("es", "pt"),
    ("fr", "pt"),
    ("gl", "en"),
    ("gl_pt", "en"),
    ("he", "pt"),
    ("it", "pt"),
    ("pt", "en"),
    ("ru", "en"),
    ("ru", "pt"),
    ("tr", "en"),
)


class TedHrlrConfig(tfds.core.BuilderConfig):
  """BuilderConfig for TED talk data comparing high/low resource languages."""

  def __init__(self, *, language_pair=(None, None), **kwargs):
    """BuilderConfig for TED talk data comparing high/low resource languages.

    The first language in `language_pair` should either be a 2-letter coded
    string or two such strings joined by an underscore (e.g., "az" or "az_tr").
    In cases where it contains two languages, the train data set will contain an
    (unlabelled) mix of the two languages and the validation and test sets
    will contain only the first language. This dataset will refer to the
    source language by the 5-letter string with the underscore. The second
    language in `language_pair` must be a 2-letter coded string.

    For example, to get pairings between Russian and English, specify
    `("ru", "en")` as `language_pair`. To get a mix of Belarusian and Russian in
    the training set and purely Belarusian in the validation and test sets,
    specify `("be_ru", "en")`.

    Args:
      language_pair: pair of languages that will be used for translation. The
        first will be used as source and second as target in supervised mode.
      **kwargs: keyword arguments forwarded to super.
    """
    name = "%s_to_%s" % (language_pair[0].replace("_", ""), language_pair[1])

    description = ("Translation dataset from %s to %s in plain text.") % (
        language_pair[0], language_pair[1])
    super(TedHrlrConfig, self).__init__(
        name=name, description=description, **kwargs)

    # Validate language pair.
    assert language_pair in _VALID_LANGUAGE_PAIRS, (
        "Config language pair (%s, "
        "%s) not supported") % language_pair

    self.language_pair = language_pair


class TedHrlrTranslate(tfds.core.GeneratorBasedBuilder):
  """TED talk data set for comparing high and low resource languages."""

  BUILDER_CONFIGS = [
      TedHrlrConfig(  # pylint: disable=g-complex-comprehension
          language_pair=pair,
          version=tfds.core.Version("1.0.0"),
          release_notes={
              "1.0.0": "New split API (https://tensorflow.org/datasets/splits)",
          },
      ) for pair in _VALID_LANGUAGE_PAIRS
  ]

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.Translation(
            languages=self.builder_config.language_pair),
        homepage="https://github.com/neulab/word-embeddings-for-nmt",
        supervised_keys=self.builder_config.language_pair,
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    dl_dir = dl_manager.download_and_extract(_DATA_URL)
    source, target = self.builder_config.language_pair

    data_dir = os.path.join(dl_dir, "datasets", "%s_to_%s" % (source, target))

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                "source_file":
                    os.path.join(data_dir, "{}.train".format(
                        source.replace("_", "-"))),
                "target_file":
                    os.path.join(data_dir, "{}.train".format(target))
            }),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={
                "source_file":
                    os.path.join(data_dir, "{}.dev".format(
                        source.split("_")[0])),
                "target_file":
                    os.path.join(data_dir, "{}.dev".format(target))
            }),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                "source_file":
                    os.path.join(data_dir, "{}.test".format(
                        source.split("_")[0])),
                "target_file":
                    os.path.join(data_dir, "{}.test".format(target))
            }),
    ]

  def _generate_examples(self, source_file, target_file):
    """This function returns the examples in the raw (text) form."""
    with tf.io.gfile.GFile(source_file) as f:
      source_sentences = f.read().split("\n")
    with tf.io.gfile.GFile(target_file) as f:
      target_sentences = f.read().split("\n")

    assert len(target_sentences) == len(
        source_sentences), "Sizes do not match: %d vs %d for %s vs %s." % (len(
            source_sentences), len(target_sentences), source_file, target_file)

    source, target = self.builder_config.language_pair
    for idx, (l1, l2) in enumerate(
        zip(source_sentences, target_sentences)):
      result = {source: l1, target: l2}
      # Make sure that both translations are non-empty.
      if all(result.values()):
        yield idx, result



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# -*- coding: utf-8 -*-
"""Tests for WMT translate dataset module."""

import os
import six
from tensorflow_datasets import testing
import tensorflow_datasets.public_api as tfds
from tensorflow_datasets.translate import wmt


class TranslateWmtCustomConfigTest(testing.DatasetBuilderTestCase):

  @classmethod
  def setUpClass(cls):
    super(TranslateWmtCustomConfigTest, cls).setUpClass()

    config = wmt.WmtConfig(
        name="small",
        language_pair=("cs", "en"),
        description="Example of custom config",
        subsets={
            "train": ["paracrawl_v3"],
            "validation": ["newstest2009", "newstest2010"],
        },
        version=tfds.core.Version("1.0.0"),
    )
    wmt.WmtTranslate.BUILDER_CONFIGS = [config]

  @classmethod
  def tearDownClass(cls):
    super(TranslateWmtCustomConfigTest, cls).tearDownClass()
    wmt.WmtTranslate.BUILDER_CONFIGS.pop()

  DATASET_CLASS = wmt.WmtTranslate
  # OVERLAPPING_SPLITS = ["validation"]

  DL_EXTRACT_RESULT = {
      "czeng17_filter": ["czeng"],
      "europarl_v9": ["sentences.cs-en.tsv"],
      "paracrawl_v3": ["sentences.cs-en.tmx"],
      "commoncrawl": ["commoncrawl"],
      "newscommentary_v14": ["sentences.cs-en.tsv"],
      "wikititles_v1": ["sentences.cs-en.tsv"],
      "newssyscomb2009": ["validation"],
      "newstest2008": ["validation"],
      "newstest2009": ["validation"],
      "newstest2010": ["validation"],
      "newstest2011": ["validation"],
      "newstest2012": ["validation"],
      "newstest2013": ["validation"],
      "newstest2014": ["validation"],
      "newstest2015": ["validation"],
      "newstest2016": ["validation"],
      "newstest2017": ["validation"],
      "newstest2018": ["validation"],
  }

  SPLITS = {
      "train": 2,
      "validation": 4,
  }

  # Wmt itself do not define checksums. Checksums are contained in individual
  # `wmt16.txt`, `wmt17.txt`,... files.
  SKIP_CHECKSUMS = True

  def test_gzip_reading(self):
    results = [
        x for _, x in wmt._parse_parallel_sentences(
            os.path.join(self.example_dir, "first.cs.gz"),
            os.path.join(self.example_dir, "second.en.txt"))
    ]
    self.assertEqual(results[1]["cs"], "zmizel")
    if six.PY3:
      self.assertEqual(results[0]["cs"], "běžím")
    else:
      self.assertTrue(results[0]["cs"] == u"běžím")  # pylint: disable=g-generic-assert


if __name__ == "__main__":
  testing.test_main()



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""CFQ (Compositional Freebase Question) dataset."""

import json
import os
import re
from absl import logging
import tensorflow.compat.v2 as tf
import tensorflow_datasets.public_api as tfds

_CITATION = """
@inproceedings{Keysers2020,
  title={Measuring Compositional Generalization: A Comprehensive Method on
         Realistic Data},
  author={Daniel Keysers and Nathanael Sch\"{a}rli and Nathan Scales and
          Hylke Buisman and Daniel Furrer and Sergii Kashubin and
          Nikola Momchev and Danila Sinopalnikov and Lukasz Stafiniak and
          Tibor Tihon and Dmitry Tsarkov and Xiao Wang and Marc van Zee and
          Olivier Bousquet},
  booktitle={ICLR},
  year={2020},
  url={https://arxiv.org/abs/1912.09713.pdf},
}
"""

_DESCRIPTION = """
The CFQ dataset (and it's splits) for measuring compositional generalization.

See https://arxiv.org/abs/1912.09713.pdf for background.

A note about the validation set: Since it has the same distribution as the test
set and we are interested in measuring the compositional generalization of a
*model* with respect to an *unknown* test distribution we suggest that any
tuning should be done on a subset of the train set only (see section 5.1 of the
paper).

Example usage:

```
data = tfds.load('cfq/mcd1')
```
"""

_DATA_URL = 'https://storage.googleapis.com/cfq_dataset/cfq.tar.gz'

_RANDOM_SEEDS = [
    '4_0', '4_42', '4.5_0', '4.5_45', '5_50', '5_50', '5.5_0', '5.5_55', '6_0'
]


class CFQConfig(tfds.core.BuilderConfig):
  """BuilderConfig for CFQ splits."""

  def __init__(self,
               *,
               name=None,
               directory=None,
               compound_divergence=None,
               random_seed=None,
               **kwargs):
    """BuilderConfig for CFQ.

    Can be constucted in two ways:
    1. With directory and name in which case these determine the split file.
    2. With compound_divergence (and optionally random_seed).

    Args:
      name: Unique name of the split.
      directory: Which subdirectory to read the split from.
      compound_divergence: The desired compound divergence.
      random_seed: The random seed. Can be either the specific random-seeds used
        to generate the split as string or an index in the range [1, 9].
      **kwargs: keyword arguments forwarded to super.
    """
    if compound_divergence is not None:
      if random_seed is None:
        random_seed_index = 1
        random_seed = _RANDOM_SEEDS[0]
      elif random_seed in range(0, 10):
        random_seed_index = random_seed
        random_seed = _RANDOM_SEEDS[random_seed - 1]
      elif random_seed in _RANDOM_SEEDS:
        random_seed_index = _RANDOM_SEEDS.index(random_seed)
      else:
        raise ValueError('Invalid random seed: %s' % random_seed)
      directory = 'splits/all_divergence_splits'
      split_name = 'divergence_split_s0.4_d%s_r%s' % (compound_divergence,
                                                      random_seed)
      name = 'cd%s_r%s' % (compound_divergence, random_seed_index)
    else:
      directory = 'splits'
      split_name = name
    super(CFQConfig, self).__init__(
        name=name,
        version=tfds.core.Version('1.2.0'),
        **kwargs)
    self.split_file = os.path.join(directory, split_name + '.json')


_QUESTION = 'question'
_QUERY = 'query'
_QUESTION_FIELD = 'questionPatternModEntities'
_QUERY_FIELD = 'sparqlPatternModEntities'


def _generate_compound_divergence_builder_configs():
  """Generate configs for different compound divergences and random seeds."""
  configs = []
  for compound_divergence in [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 1]:
    for random_seed in range(1, 10):
      configs.append(
          CFQConfig(
              compound_divergence=compound_divergence, random_seed=random_seed))
  return configs


class CFQ(tfds.core.GeneratorBasedBuilder):
  """CFQ task / splits."""

  BUILDER_CONFIGS = [
      CFQConfig(name='mcd1'),
      CFQConfig(name='mcd2'),
      CFQConfig(name='mcd3'),
      CFQConfig(name='question_complexity_split'),
      CFQConfig(name='question_pattern_split'),
      CFQConfig(name='query_complexity_split'),
      CFQConfig(name='query_pattern_split'),
      CFQConfig(name='random_split'),
  ] + _generate_compound_divergence_builder_configs()

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            _QUESTION: tfds.features.Text(),
            _QUERY: tfds.features.Text(),
        }),
        supervised_keys=(_QUESTION, _QUERY),
        homepage='https://github.com/google-research/google-research/tree/master/cfq',
        citation=_CITATION,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    data_dir = dl_manager.download_and_extract(_DATA_URL)
    data_dir = os.path.join(data_dir, 'cfq')
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                'base_directory': data_dir,
                'splits_file': self.builder_config.split_file,
                'split_id': 'trainIdxs'
            }),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={
                'base_directory': data_dir,
                'splits_file': self.builder_config.split_file,
                'split_id': 'devIdxs'
            }),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                'base_directory': data_dir,
                'splits_file': self.builder_config.split_file,
                'split_id': 'testIdxs'
            })
    ]

  def _scrub_json(self, content):
    """Reduce JSON by filtering out only the fields of interest."""
    # Loading of json data with the standard Python library is very inefficient:
    # For the 4GB dataset file it requires more than 40GB of RAM and takes 3min.
    # There are more efficient libraries but in order to avoid additional
    # dependencies we use a simple (perhaps somewhat brittle) regexp to reduce
    # the content to only what is needed. This takes 1min to execute but
    # afterwards loading requires only 500MB or RAM and is done in 2s.
    regex = re.compile(
        r'("%s":\s*"[^"]*").*?("%s":\s*"[^"]*")' %
        (_QUESTION_FIELD, _QUERY_FIELD), re.DOTALL)
    return '[' + ','.join([
        '{' + m.group(1) + ',' + m.group(2) + '}'
        for m in regex.finditer(content)
    ]) + ']'

  def _generate_examples(self, base_directory, splits_file, split_id):
    """Yields examples."""
    samples_path = os.path.join(base_directory, 'dataset.json')
    splits_path = os.path.join(base_directory, splits_file)
    with tf.io.gfile.GFile(samples_path) as samples_file:
      with tf.io.gfile.GFile(splits_path) as splits_file:
        logging.info('Reading json from %s into memory...', samples_path)
        samples = json.loads(self._scrub_json(samples_file.read()))
        logging.info('%d samples loaded', len(samples))
        logging.info('Loaded json data from %s.', samples_path)
        splits = json.load(splits_file)
        for idx in splits[split_id]:
          sample = samples[idx]
          yield idx, {
              _QUESTION: sample[_QUESTION_FIELD],
              _QUERY: sample[_QUERY_FIELD]
          }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

r"""Script which generates datasets dataframes HTML.

"""

import functools

from absl import flags
import pandas

import tensorflow as tf
import tensorflow_datasets as tfds
from tensorflow_datasets.scripts.documentation import script_utils

FLAGS = flags.FLAGS

flags.DEFINE_string(
    'datasets', None,
    'Comma separated list of datasets to generates. None for all datasets.')
flags.DEFINE_string(
    'dst_dir', tfds.core.gcs_path('visualization/dataframe'),
    'Destination dir to save the dataframe html.')
flags.DEFINE_boolean(
    'overwrite', False, 'If True, overwrite the existing visualizations.')


def _save_html(dst_path: str, df: pandas.DataFrame) -> None:
  with tf.io.gfile.GFile(dst_path, 'w') as f:
    f.write(df._repr_html_())  # pylint: disable=protected-access


def main(_):
  """Main script."""
  datasets = FLAGS.datasets.split(',') if FLAGS.datasets else None
  generate_and_save_dataframe_fn = functools.partial(
      script_utils.generate_and_save_artifact,
      dst_dir=FLAGS.dst_dir,
      overwrite=FLAGS.overwrite,
      file_extension='.html',
      get_artifact_fn=tfds.as_dataframe,
      save_artifact_fn=_save_html,
  )
  script_utils.multi_process_map(
      worker_fn=generate_and_save_dataframe_fn,
      datasets=datasets,
  )


if __name__ == '__main__':
  flags.mark_flags_as_required(['dst_dir'])
  script_utils.multi_process_run(main)



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for corrupted_imagenet."""

from tensorflow_datasets import testing
from tensorflow_datasets.image_classification import imagenet2012_corrupted


class Imagenet2012CorruptedTest(testing.DatasetBuilderTestCase):
  BUILDER_CONFIG_NAMES_TO_TEST = [  # pylint: disable=g-long-ternary
      "gaussian_noise_1", "shot_noise_2", "impulse_noise_3", "defocus_blur_4",
      "glass_blur_5", "motion_blur_1", "zoom_blur_2", "snow_3", "frost_4",
      "fog_5", "brightness_1", "contrast_2", "elastic_transform_3",
      "pixelate_4", "jpeg_compression_5", "gaussian_blur_1", "saturate_2",
      "spatter_3", "speckle_noise_4"
  ]

  DATASET_CLASS = imagenet2012_corrupted.Imagenet2012Corrupted
  SPLITS = {  # Expected number of examples on the train/validation splits.
      "validation": 10,
  }
  DL_EXTRACT_RESULT = [
      "ILSVRC2012_img_train.tar",
      "ILSVRC2012_img_val.tar",
  ]
  DL_DOWNLOAD_RESULT = [
      "frost1.png", "frost2.png", "frost3.png", "frost4.jpg", "frost5.jpg",
      "frost6.jpg"
  ]


if __name__ == "__main__":
  testing.test_main()



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for MIT Scene Parsing Benchmark (SceneParse150)."""

from tensorflow_datasets import testing
from tensorflow_datasets.image import scene_parse_150


class SceneParse150Test(testing.DatasetBuilderTestCase):

  DATASET_CLASS = scene_parse_150.SceneParse150

  SPLITS = {
      "train": 3,
      "test": 3,
  }

  DL_EXTRACT_RESULT = {
      "images": "images",
      "annotations": "annotations",
      "test": "test",
  }


if __name__ == "__main__":
  testing.test_main()
